@@ -14,6 +14,7 @@ DROP TABLE IF EXISTS g_user_session_scheme;
 DROP TABLE IF EXISTS g_scope;
 DROP TABLE IF EXISTS g_plugin_module_instance;
 DROP TABLE IF EXISTS g_user_module_instance;
+DROP TABLE IF EXISTS g_user_middleware_module_instance;
 DROP TABLE IF EXISTS g_user_auth_scheme_module_instance;
 DROP TABLE IF EXISTS g_client_module_instance;
 DROP TABLE IF EXISTS g_user_session;
@@ -30,6 +31,16 @@ CREATE TABLE g_user_module_instance (
   gumi_enabled INTEGER DEFAULT 1
 );
 
+CREATE TABLE g_user_middleware_module_instance (
+  gummi_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gummi_module TEXT NOT NULL,
+  gummi_order INTEGER NOT NULL,
+  gummi_name TEXT NOT NULL,
+  gummi_display_name TEXT DEFAULT '',
+  gummi_parameters TEXT,
+  gummi_enabled INTEGER DEFAULT 1
+);
+
 CREATE TABLE g_user_auth_scheme_module_instance (
   guasmi_id INTEGER PRIMARY KEY AUTOINCREMENT,
   guasmi_module TEXT NOT NULL,
