@@ -2,9 +2,9 @@
  *
  * Glewlwyd SSO Access Token token check
  *
- * Copyright 2016-2020 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2016-2021 Nicolas Mora <mail@babelouest.org>
  *
- * Version 20201013
+ * Version 20211029
  *
  * The MIT License (MIT)
  * 
@@ -134,21 +134,19 @@ static int access_token_check_validity(struct _oidc_resource_config * config, js
  */
 static json_t * access_token_check_signature(struct _oidc_resource_config * config, const char * token_value) {
   json_t * j_return = NULL, * j_grants;
-  jwt_t * jwt = r_jwt_copy(config->jwt);
+  jwt_t * jwt = NULL;
   jwk_t * jwk = NULL;
   const char * kid;
   
   if (token_value != NULL) {
-    if (r_jwt_parse(jwt, token_value, 0) == RHN_OK) {
+    if ((jwt = r_jwt_quick_parse(token_value, R_PARSE_NONE, config->x5u_flags)) != NULL) {
       if ((kid = r_jwt_get_header_str_value(jwt, "kid")) != NULL) {
-        if ((jwk = r_jwks_get_by_kid(jwt->jwks_pubkey_sign, kid)) == NULL) {
-          j_return = json_pack("{si}", "result", G_TOKEN_ERROR_INVALID_TOKEN);
-        }
+        jwk = r_jwks_get_by_kid(config->jwks_public, kid);
       } else {
-        jwk = r_jwk_copy(config->jwk_verify_default);
+        jwk = r_jwks_get_at(config->jwks_public, 0);
       }
-      if (j_return == NULL) {
-        if (r_jwt_verify_signature(jwt, jwk, 0) == RHN_OK && r_jwt_get_sign_alg(jwt) == config->alg) {
+      if (jwk != NULL) {
+        if (r_jwt_verify_signature(jwt, jwk, 0) == RHN_OK) {
           j_grants = r_jwt_get_full_claims_json_t(jwt);
           if (j_grants != NULL) {
             j_return = json_pack("{siso}", "result", G_TOKEN_OK, "grants", j_grants);
@@ -158,15 +156,15 @@ static json_t * access_token_check_signature(struct _oidc_resource_config * conf
         } else {
           j_return = json_pack("{si}", "result", G_TOKEN_ERROR_INVALID_TOKEN);
         }
+        r_jwk_free(jwk);
       }
-      r_jwk_free(jwk);
     } else {
       j_return = json_pack("{si}", "result", G_TOKEN_ERROR_INVALID_TOKEN);
     }
+    r_jwt_free(jwt);
   } else {
     j_return = json_pack("{si}", "result", G_TOKEN_ERROR_INVALID_TOKEN);
   }
-  r_jwt_free(jwt);
   return j_return;
 }
 
@@ -213,19 +211,19 @@ int callback_check_glewlwyd_oidc_access_token (const struct _u_request * request
             u_map_put(response->map_header, HEADER_RESPONSE, response_value);
             o_free(response_value);
           } else {
-            res = U_CALLBACK_CONTINUE;
-            response->shared_data = (void*)json_pack("{sssOsO*}", "sub", json_string_value(json_object_get(json_object_get(j_access_token, "grants"), "sub")), "scope", json_object_get(j_res_scope, "scope"), "jkt", json_object_get(json_object_get(json_object_get(j_access_token, "grants"), "cnf"), "jkt"));
-            if (json_object_get(json_object_get(j_access_token, "grants"), "aud") != NULL) {
-              json_object_set((void*)response->shared_data, "aud", json_object_get(json_object_get(j_access_token, "grants"), "aud"));
-            }
-            if (json_object_get(json_object_get(j_access_token, "grants"), "client_id") != NULL) {
-              json_object_set((void*)response->shared_data, "client_id", json_object_get(json_object_get(j_access_token, "grants"), "client_id"));
-            }
-            if (json_object_get(json_object_get(j_access_token, "grants"), "claims") != NULL) {
-              json_object_set((void*)response->shared_data, "claims", json_object_get(json_object_get(j_access_token, "grants"), "claims"));
-            }
-            if (response->shared_data == NULL) {
+            if (ulfius_set_response_shared_data(response, json_pack("{ss?sOsO*}", "sub", json_string_value(json_object_get(json_object_get(j_access_token, "grants"), "sub")), "scope", json_object_get(j_res_scope, "scope"), "jkt", json_object_get(json_object_get(json_object_get(j_access_token, "grants"), "cnf"), "jkt")), (void (*)(void *))&json_decref) != U_OK) {
               res = U_CALLBACK_ERROR;
+            } else {
+              res = U_CALLBACK_CONTINUE;
+              if (json_object_get(json_object_get(j_access_token, "grants"), "aud") != NULL) {
+                json_object_set((json_t *)response->shared_data, "aud", json_object_get(json_object_get(j_access_token, "grants"), "aud"));
+              }
+              if (json_object_get(json_object_get(j_access_token, "grants"), "client_id") != NULL) {
+                json_object_set((json_t *)response->shared_data, "client_id", json_object_get(json_object_get(j_access_token, "grants"), "client_id"));
+              }
+              if (json_object_get(json_object_get(j_access_token, "grants"), "claims") != NULL) {
+                json_object_set((json_t *)response->shared_data, "claims", json_object_get(json_object_get(j_access_token, "grants"), "claims"));
+              }
             }
           }
           json_decref(j_res_scope);
@@ -256,7 +254,7 @@ int callback_check_glewlwyd_oidc_access_token (const struct _u_request * request
 /**
  * Parse the DPoP header and extract its jkt value if the DPoP is valid
  */
-json_t * verify_dpop_proof(const struct _u_request * request, const char * htm, const char * htu, time_t max_iat, const char * jkt) {
+json_t * verify_dpop_proof(const struct _u_request * request, const char * htm, const char * htu, time_t max_iat, const char * jkt, const char * access_token) {
   json_t * j_return = NULL, * j_header = NULL, * j_claims = NULL;
   const char * dpop_header;
   jwt_t * dpop_jwt = NULL;
@@ -264,6 +262,9 @@ json_t * verify_dpop_proof(const struct _u_request * request, const char * htm,
   jwk_t * jwk_header = NULL;
   char * jkt_from_token = NULL;
   time_t now;
+  unsigned char ath[32] = {0}, ath_enc[64] = {0};
+  size_t ath_len = 32, ath_enc_len = 64;
+  gnutls_datum_t hash_data;
   
   if ((dpop_header = u_map_get_case(request->map_header, HEADER_DPOP)) != NULL) {
     if (r_jwt_init(&dpop_jwt) == RHN_OK) {
@@ -276,7 +277,9 @@ json_t * verify_dpop_proof(const struct _u_request * request, const char * htm,
               break;
             }
             if ((alg = r_jwt_get_sign_alg(dpop_jwt)) != R_JWA_ALG_RS256 && alg != R_JWA_ALG_RS384 && alg != R_JWA_ALG_RS512 &&
-                alg != R_JWA_ALG_ES256 && alg != R_JWA_ALG_ES384 && alg != R_JWA_ALG_ES512 && alg != R_JWA_ALG_EDDSA && alg != R_JWA_ALG_ES256K) {
+                alg != R_JWA_ALG_ES256 && alg != R_JWA_ALG_ES384 && alg != R_JWA_ALG_ES512 && 
+                alg != R_JWA_ALG_PS256 && alg != R_JWA_ALG_PS384 && alg != R_JWA_ALG_PS512 &&
+                alg != R_JWA_ALG_EDDSA && alg != R_JWA_ALG_ES256K) {
               y_log_message(Y_LOG_LEVEL_DEBUG, "verify_dpop_proof - Invalid sign_alg");
               j_return = json_pack("{si}", "result", G_TOKEN_ERROR_INVALID_TOKEN);
               break;
@@ -327,6 +330,23 @@ json_t * verify_dpop_proof(const struct _u_request * request, const char * htm,
               j_return = json_pack("{si}", "result", G_TOKEN_ERROR_INVALID_TOKEN);
               break;
             }
+            hash_data.data = (unsigned char*)access_token;
+            hash_data.size = o_strlen(access_token);
+            if (gnutls_fingerprint(GNUTLS_DIG_SHA256, &hash_data, ath, &ath_len) != GNUTLS_E_SUCCESS) {
+              y_log_message(Y_LOG_LEVEL_ERROR, "verify_dpop_proof - Error gnutls_fingerprint");
+              j_return = json_pack("{si}", "result", G_TOKEN_ERROR);
+              break;
+            }
+            if (!o_base64url_encode(ath, ath_len, ath_enc, &ath_enc_len)) {
+              y_log_message(Y_LOG_LEVEL_ERROR, "verify_dpop_proof - Error o_base64url_encode ath");
+              j_return = json_pack("{si}", "result", G_TOKEN_ERROR);
+              break;
+            }
+            if (0 != o_strcmp((const char *)ath_enc, r_jwt_get_claim_str_value(dpop_jwt, "ath"))) {
+              y_log_message(Y_LOG_LEVEL_ERROR, "verify_dpop_proof - Error ath invalid");
+              j_return = json_pack("{si}", "result", G_TOKEN_ERROR_INVALID_TOKEN);
+              break;
+            }
             if ((jkt_from_token = r_jwk_thumbprint(jwk_header, R_JWK_THUMB_SHA256, R_FLAG_IGNORE_REMOTE)) == NULL) {
               y_log_message(Y_LOG_LEVEL_ERROR, "verify_dpop_proof - Error r_jwk_thumbprint");
               j_return = json_pack("{si}", "result", G_TOKEN_ERROR);
