@@ -29,18 +29,22 @@
 #include <yder.h>
 #include <orcania.h>
 #include <hoel.h>
-#include "../glewlwyd-common.h"
+#include "glewlwyd-common.h"
 
 #define G_TABLE_CLIENT "g_client"
 #define G_TABLE_CLIENT_SCOPE "g_client_scope"
 #define G_TABLE_CLIENT_SCOPE_CLIENT "g_client_scope_client"
 #define G_TABLE_CLIENT_PROPERTY "g_client_property"
 
+#define G_PBKDF2_ITERATOR_SEP ','
+
 struct mod_parameters {
-  int use_glewlwyd_connection;
-  digest_algorithm hash_algorithm;
+  int                    use_glewlwyd_connection;
+  digest_algorithm       hash_algorithm;
   struct _h_connection * conn;
-  json_t * j_params;
+  json_t               * j_params;
+  unsigned int           PBKDF2_iterations;
+  struct config_module * config_glewlwyd;
 };
 
 static json_t * is_client_database_parameters_valid(json_t * j_params) {
@@ -65,8 +69,8 @@ static json_t * is_client_database_parameters_valid(json_t * j_params) {
           if (json_object_get(j_params, "mariadb-host") == NULL || !json_is_string(json_object_get(j_params, "mariadb-host"))) {
             json_array_append_new(j_error, json_string("mariadb-host is mandatory and must be a string"));
           }
-          if (json_object_get(j_params, "mariadb-client") == NULL || !json_is_string(json_object_get(j_params, "mariadb-client"))) {
-            json_array_append_new(j_error, json_string("mariadb-client is mandatory and must be a string"));
+          if (json_object_get(j_params, "mariadb-user") == NULL || !json_is_string(json_object_get(j_params, "mariadb-user"))) {
+            json_array_append_new(j_error, json_string("mariadb-user is mandatory and must be a string"));
           }
           if (json_object_get(j_params, "mariadb-password") == NULL || !json_is_string(json_object_get(j_params, "mariadb-password"))) {
             json_array_append_new(j_error, json_string("mariadb-password is mandatory and must be a string"));
@@ -74,7 +78,7 @@ static json_t * is_client_database_parameters_valid(json_t * j_params) {
           if (json_object_get(j_params, "mariadb-dbname") == NULL || !json_is_string(json_object_get(j_params, "mariadb-dbname"))) {
             json_array_append_new(j_error, json_string("mariadb-dbname is mandatory and must be a string"));
           }
-          if (json_object_get(j_params, "mariadb-port") != NULL && (!json_is_integer(json_object_get(j_params, "mariadb-dbname")) || json_integer_value(json_object_get(j_params, "mariadb-dbname")) < 0)) {
+          if (json_object_get(j_params, "mariadb-port") != NULL && (!json_is_integer(json_object_get(j_params, "mariadb-port")) || json_integer_value(json_object_get(j_params, "mariadb-port")) < 0)) {
             json_array_append_new(j_error, json_string("mariadb-port is optional and must be a positive integer (default: 0)"));
           }
         } else if (0 == o_strcmp("postgre", json_string_value(json_object_get(j_params, "connection-type")))) {
@@ -107,6 +111,9 @@ static json_t * is_client_database_parameters_valid(json_t * j_params) {
           }
         }
       }
+      if (json_object_get(j_params, "pbkdf2-iterations") != NULL && json_integer_value(json_object_get(j_params, "pbkdf2-iterations")) <= 0) {
+        json_array_append_new(j_error, json_string("pbkdf2-iterations is optional and must be a positive non null integer"));
+      }
     }
     if (json_array_size(j_error)) {
       j_return = json_pack("{sisO}", "result", G_ERROR_PARAM, "error", j_error);
@@ -227,6 +234,7 @@ static int append_client_properties(struct mod_parameters * param, json_t * j_cl
       json_decref(j_result);
     } else {
       y_log_message(Y_LOG_LEVEL_ERROR, "append_client_properties database - Error executing j_query");
+      param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
       ret = G_ERROR_DB;
     }
   } else {
@@ -280,6 +288,7 @@ static int append_client_properties(struct mod_parameters * param, json_t * j_cl
       json_decref(j_result);
     } else {
       y_log_message(Y_LOG_LEVEL_ERROR, "append_client_properties database - Error executing j_query");
+      param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
       ret = G_ERROR_DB;
     }
   }
@@ -323,6 +332,7 @@ static json_t * database_client_scope_get(struct mod_parameters * param, json_in
     json_decref(j_result);
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "database_client_scope_get database - Error executing j_query");
+    param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     j_return = json_pack("{si}", "result", G_ERROR_DB);
   }
   return j_return;
@@ -381,6 +391,7 @@ static json_t * database_client_get(const char * client_id, void * cls, int prof
     json_decref(j_result);
   } else {
     j_return = json_pack("{si}", "result", G_ERROR_DB);
+    param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     y_log_message(Y_LOG_LEVEL_ERROR, "database_client_get database - Error executing j_query");
   }
   return j_return;
@@ -392,8 +403,8 @@ static char * get_password_clause_write(struct mod_parameters * param, const cha
   if (!o_strlen(password)) {
     clause = o_strdup("''");
   } else if (param->conn->type == HOEL_DB_TYPE_SQLITE) {
-    if (generate_digest_pbkdf2(password, NULL, digest)) {
-      clause = msprintf("'%s'", digest);
+    if (generate_digest_pbkdf2(password, param->PBKDF2_iterations, NULL, digest)) {
+      clause = msprintf("'%s%c%u'", digest, G_PBKDF2_ITERATOR_SEP, param->PBKDF2_iterations);
     } else {
       y_log_message(Y_LOG_LEVEL_ERROR, "get_password_clause_write database - Error generate_digest_pbkdf2");
     }
@@ -417,12 +428,12 @@ static char * get_password_clause_write(struct mod_parameters * param, const cha
   return clause;
 }
 
-static char * get_salt_from_password_hash(struct mod_parameters * param, const char * client_id) {
+static char * get_salt_from_password_hash(struct mod_parameters * param, const char * client_id, unsigned int * iterations) {
   json_t * j_query, * j_result;
   int res;
   unsigned char password_b64_decoded[1024] = {0};
-  char * salt = NULL, * client_id_escaped, * client_id_clause;
-  size_t password_b64_decoded_len;
+  char * salt = NULL, * client_id_escaped, * client_id_clause, * str_iterator;
+  size_t password_b64_decoded_len, gc_password_len;
   
   client_id_escaped = h_escape_string_with_quotes(param->conn, client_id);
   client_id_clause = msprintf(" = UPPER(%s)", client_id_escaped);
@@ -443,7 +454,13 @@ static char * get_salt_from_password_hash(struct mod_parameters * param, const c
   json_decref(j_query);
   if (res == H_OK) {
     if (json_array_size(j_result) && json_string_length(json_object_get(json_array_get(j_result, 0), "gc_password"))) {
-      if (o_base64_decode((const unsigned char *)json_string_value(json_object_get(json_array_get(j_result, 0), "gc_password")), json_string_length(json_object_get(json_array_get(j_result, 0), "gc_password")), password_b64_decoded, &password_b64_decoded_len)) {
+      if ((str_iterator = o_strchr(json_string_value(json_object_get(json_array_get(j_result, 0), "gc_password")), G_PBKDF2_ITERATOR_SEP)) != NULL) {
+        gc_password_len = o_strchr(json_string_value(json_object_get(json_array_get(j_result, 0), "gc_password")), G_PBKDF2_ITERATOR_SEP) - json_string_value(json_object_get(json_array_get(j_result, 0), "gc_password"));
+        *iterations = (unsigned int)strtol(str_iterator+1, NULL, 10);
+      } else {
+        gc_password_len = json_string_length(json_object_get(json_array_get(j_result, 0), "gc_password"));
+      }
+      if (o_base64_decode((const unsigned char *)json_string_value(json_object_get(json_array_get(j_result, 0), "gc_password")), gc_password_len, password_b64_decoded, &password_b64_decoded_len)) {
         if ((salt = o_strdup((const char *)password_b64_decoded + password_b64_decoded_len - GLEWLWYD_DEFAULT_SALT_LENGTH)) == NULL) {
           y_log_message(Y_LOG_LEVEL_ERROR, "get_salt_from_password_hash - Error extracting salt");
         }
@@ -462,11 +479,16 @@ static char * get_salt_from_password_hash(struct mod_parameters * param, const c
 
 static char * get_password_clause_check(struct mod_parameters * param, const char * client_id, const char * password) {
   char * clause = NULL, * password_encoded, digest[1024] = {0}, * salt;
+  unsigned int iterations = 0;
   
   if (param->conn->type == HOEL_DB_TYPE_SQLITE) {
-    if ((salt = get_salt_from_password_hash(param, client_id)) != NULL) {
-      if (generate_digest_pbkdf2(password, salt, digest)) {
-        clause = msprintf(" = '%s'", digest);
+    if ((salt = get_salt_from_password_hash(param, client_id, &iterations)) != NULL) {
+      if (generate_digest_pbkdf2(password, iterations?iterations:1000, salt, digest)) {
+        if (iterations) {
+          clause = msprintf(" = '%s%c%u'", digest, G_PBKDF2_ITERATOR_SEP, iterations);
+        } else {
+          clause = msprintf(" = '%s'", digest);
+        }
       } else {
         y_log_message(Y_LOG_LEVEL_ERROR, "get_password_clause_check database - Error generate_digest_pbkdf2");
       }
@@ -560,6 +582,7 @@ static int save_client_properties(struct mod_parameters * param, json_t * j_clie
           ret = G_OK;
         } else {
           y_log_message(Y_LOG_LEVEL_ERROR, "insert_client_properties database - Error executing j_query insert");
+          param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
           ret = G_ERROR_DB;
         }
       } else {
@@ -567,6 +590,7 @@ static int save_client_properties(struct mod_parameters * param, json_t * j_clie
       }
     } else {
       y_log_message(Y_LOG_LEVEL_ERROR, "insert_client_properties database - Error executing j_query delete");
+      param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
       ret = G_ERROR_DB;
     }
     json_decref(j_array);
@@ -673,6 +697,7 @@ static int save_client_scope(struct mod_parameters * param, json_t * j_scope, js
     }
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "save_client_scope database - Error executing j_query delete");
+    param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     ret = G_ERROR_DB;
   }
   
@@ -704,6 +729,7 @@ json_t * client_module_init(struct config_module * config, int readonly, json_t
     if (*cls != NULL) {
       ((struct mod_parameters *)*cls)->j_params = json_incref(j_parameters);
       ((struct mod_parameters *)*cls)->hash_algorithm = config->hash_algorithm;
+      ((struct mod_parameters *)*cls)->config_glewlwyd = config;
       if (json_object_get(j_parameters, "use-glewlwyd-connection") != json_false()) {
           ((struct mod_parameters *)*cls)->use_glewlwyd_connection = 0;
           ((struct mod_parameters *)*cls)->conn = config->conn;
@@ -717,6 +743,11 @@ json_t * client_module_init(struct config_module * config, int readonly, json_t
           ((struct mod_parameters *)*cls)->conn = h_connect_pgsql(json_string_value(json_object_get(j_parameters, "postgre-conninfo")));
         }
       }
+      if (json_object_get(j_parameters, "pbkdf2-iterations") != NULL) {
+        ((struct mod_parameters *)*cls)->PBKDF2_iterations = (unsigned int)json_integer_value(json_object_get(j_parameters, "pbkdf2-iterations"));
+      } else {
+        ((struct mod_parameters *)*cls)->PBKDF2_iterations = G_PBKDF2_ITERATOR_DEFAULT;
+      }
       if (((struct mod_parameters *)*cls)->conn != NULL) {
         j_return = json_pack("{si}", "result", G_OK);
       } else {
@@ -749,6 +780,7 @@ int client_module_close(struct config_module * config, void * cls) {
     if (((struct mod_parameters *)cls)->use_glewlwyd_connection) {
       if (h_close_db(((struct mod_parameters *)cls)->conn) != H_OK) {
         y_log_message(Y_LOG_LEVEL_ERROR, "client_module_close database - Error h_close_db");
+        config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
         ret = G_ERROR_DB;
       } else {
         ret = G_OK;
@@ -847,6 +879,7 @@ json_t * client_module_get_list(struct config_module * config, const char * patt
     json_decref(j_result);
   } else {
     j_return = json_pack("{si}", "result", G_ERROR_DB);
+    param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     y_log_message(Y_LOG_LEVEL_ERROR, "client_module_get_list database - Error executing j_query");
   }
   return j_return;
@@ -985,9 +1018,11 @@ int client_module_add(struct config_module * config, json_t * j_client, void * c
     j_gc_id = h_last_insert_id(param->conn);
     if (save_client_properties(param, j_client, json_integer_value(j_gc_id)) != G_OK) {
       y_log_message(Y_LOG_LEVEL_ERROR, "client_module_add database - Error save_client_properties");
+      param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
       ret = G_ERROR_DB;
     } else if (json_object_get(j_client, "scope") != NULL && save_client_scope(param, json_object_get(j_client, "scope"), json_integer_value(j_gc_id)) != G_OK) {
       y_log_message(Y_LOG_LEVEL_ERROR, "client_module_add database - Error save_client_scope");
+      param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
       ret = G_ERROR_DB;
     } else {
       ret = G_OK;
@@ -995,6 +1030,7 @@ int client_module_add(struct config_module * config, json_t * j_client, void * c
     json_decref(j_gc_id);
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "client_module_add database - Error executing j_query insert");
+    param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     ret = G_ERROR_DB;
   }
   return ret;
@@ -1050,15 +1086,18 @@ int client_module_update(struct config_module * config, const char * client_id,
     if (res == H_OK) {
       if (save_client_properties(param, j_client, json_integer_value(json_object_get(json_array_get(j_result, 0), "gc_id"))) != G_OK) {
         y_log_message(Y_LOG_LEVEL_ERROR, "client_module_add database - Error save_client_properties");
+        param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
         ret = G_ERROR_DB;
       } else if (json_object_get(j_client, "scope") != NULL && save_client_scope(param, json_object_get(j_client, "scope"), json_integer_value(json_object_get(json_array_get(j_result, 0), "gc_id"))) != G_OK) {
         y_log_message(Y_LOG_LEVEL_ERROR, "client_module_add database - Error save_client_scope");
+        param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
         ret = G_ERROR_DB;
       } else {
         ret = G_OK;
       }
     } else {
       y_log_message(Y_LOG_LEVEL_ERROR, "client_module_add database - Error executing j_query update");
+      param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
       ret = G_ERROR_DB;
     }
   } else {
@@ -1094,6 +1133,7 @@ int client_module_delete(struct config_module * config, const char * client_id,
     ret = G_OK;
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "client_module_delete database - Error executing j_query");
+    param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     ret = G_ERROR_DB;
   }
   return ret;
@@ -1139,6 +1179,7 @@ int client_module_check_password(struct config_module * config, const char * cli
     json_decref(j_result);
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "client_module_check_password database - Error executing j_query");
+    param->config_glewlwyd->glewlwyd_module_callback_metrics_increment_counter(param->config_glewlwyd, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     ret = G_ERROR_DB;
   }
   return ret;
