@@ -10,7 +10,25 @@
  * 
  * Copyright 2016-2020 Nicolas Mora <mail@babelouest.org>
  *
- * License MIT
+ * The MIT License (MIT)
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  *
  */
 
@@ -18,7 +36,7 @@
 #include <jansson.h>
 #include <yder.h>
 #include <orcania.h>
-#include "../glewlwyd-common.h"
+#include "glewlwyd-common.h"
 
 /**
  *
@@ -187,7 +205,7 @@ json_t * client_module_init(struct config_module * config, int readonly, json_t
     if (json_string_length(json_object_get(j_parameters, "client-id-prefix"))) {
       prefix = json_string_value(json_object_get(j_parameters, "client-id-prefix"));
     }
-    *cls = (void*)json_pack("[{ss+ ss ss so s[ssssss] s[sss] ss s[] so}{ss+ ss ss so s[s] s[s] s[] so}{ss+ ss ss so ss s[ssssssss] s[ss] ss s[ss] so}{ss+ ss ss so ss s[ssss] s[s] ss so}]",
+    *cls = (void*)json_pack("[{ss+ ss ss so s[ssssss] s[sss] ss s[] so}{ss+ ss ss so s[s] s[s] s[] so}{ss+ ss ss so ss s[ssssssss] s[ss] ss s[ss] so}{ss+ ss ss so ss s[ssss] s[s] ss so ss}]",
                               "client_id",
                               prefix,
                               "client1_id",
@@ -280,7 +298,9 @@ json_t * client_module_init(struct config_module * config, int readonly, json_t
                               "sector_identifier_uri",
                                 "https://sector4.glewlwyd.tld",
                               "enabled",
-                              json_true());
+                              json_true(),
+                              "request_object_signing_alg",
+                              "HS256");
     y_log_message(Y_LOG_LEVEL_DEBUG, "client_module_init - success %s %s, prefix: '%s'", config->profile_scope, config->admin_scope, prefix);
     j_return = json_pack("{si}", "result", G_OK);
   } else {
@@ -372,7 +392,7 @@ size_t client_module_count_total(struct config_module * config, const char * pat
  */
 json_t * client_module_get_list(struct config_module * config, const char * pattern, size_t offset, size_t limit, void * cls) {
   UNUSED(config);
-  json_t * j_user = NULL, * j_array, * j_array_pattern, * j_return;
+  json_t * j_user = NULL, * j_array, * j_array_pattern, * j_return, * j_user_copy;
   size_t index = 0, counter = 0;
 
   if (limit > 0) {
@@ -390,7 +410,9 @@ json_t * client_module_get_list(struct config_module * config, const char * patt
     if (j_array != NULL) {
       json_array_foreach(j_array_pattern, index, j_user) {
         if (index >= offset && (offset + counter) < json_array_size(j_array_pattern) && counter < limit) {
-          json_array_append(j_array, j_user);
+          j_user_copy = json_deep_copy(j_user);
+          json_object_del(j_user_copy, "password");
+          json_array_append_new(j_array, j_user_copy);
           counter++;
         }
       }
