@@ -5,23 +5,30 @@
  * Authentiation server
  * Users are authenticated via various backend available: database, ldap
  * Using various authentication methods available: password, OTP, send code, etc.
- * 
+ *
  * Declarations for common constants and prototypes used in Glewlwyd main program and modules
  *
  * Copyright 2016-2021 Nicolas Mora <mail@babelouest.org>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU GENERAL PUBLIC LICENSE
- * License as published by the Free Software Foundation;
- * version 3 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU GENERAL PUBLIC LICENSE for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * The MIT License (MIT)
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  *
  */
 
@@ -54,13 +61,14 @@
 /**
  * Callback priority
  */
-#define GLEWLWYD_CALLBACK_PRIORITY_ZERO           0
-#define GLEWLWYD_CALLBACK_PRIORITY_AUTHENTICATION 1
-#define GLEWLWYD_CALLBACK_PRIORITY_APPLICATION    2
-#define GLEWLWYD_CALLBACK_PRIORITY_COMPRESSION    3
-#define GLEWLWYD_CALLBACK_PRIORITY_CLOSE          4
-#define GLEWLWYD_CALLBACK_PRIORITY_PLUGIN         5
-#define GLEWLWYD_CALLBACK_PRIORITY_FILE           100
+#define GLEWLWYD_CALLBACK_PRIORITY_ZERO            0
+#define GLEWLWYD_CALLBACK_PRIORITY_AUTHENTICATION  1
+#define GLEWLWYD_CALLBACK_PRIORITY_PRE_APPLICATION 2
+#define GLEWLWYD_CALLBACK_PRIORITY_APPLICATION     3
+#define GLEWLWYD_CALLBACK_PRIORITY_COMPRESSION     4
+#define GLEWLWYD_CALLBACK_PRIORITY_PLUGIN          5
+#define GLEWLWYD_CALLBACK_PRIORITY_FILE            100
+#define GLEWLWYD_CALLBACK_PRIORITY_POST_FILE       101
 
 /**
  * Modes available when adding or modifying a user
@@ -87,6 +95,8 @@
 
 #define GLEWLWYD_DEFAULT_SALT_LENGTH 16
 
+#define G_PBKDF2_ITERATOR_DEFAULT 150000
+
 #define SWITCH_DB_TYPE(T, M, S, P) \
         ((T)==HOEL_DB_TYPE_MARIADB?\
            (M):\
@@ -151,7 +161,27 @@ struct _user_module {
   int      (* user_module_delete)(struct config_module * config, const char * username, void * cls);
   int      (* user_module_check_password)(struct config_module * config, const char * username, const char * password, void * cls);
   int      (* user_module_update_password)(struct config_module * config, const char * username, const char ** new_passwords, size_t new_passwords_len, void * cls);
-  };
+};
+
+/**
+ * Structure used to store a user middleware module
+ */
+struct _user_middleware_module {
+  void      * file_handle;
+  char      * name;
+  char      * display_name;
+  char      * description;
+  double      api_version;
+  json_t * (* user_middleware_module_load)(struct config_module * config);
+  int      (* user_middleware_module_unload)(struct config_module * config);
+  json_t * (* user_middleware_module_init)(struct config_module * config, json_t * j_parameters, void ** cls);
+  int      (* user_middleware_module_close)(struct config_module * config, void * cls);
+  int      (* user_middleware_module_get_list)(struct config_module * config, json_t * j_user_list, void * cls);
+  int      (* user_middleware_module_get)(struct config_module * config, const char * username, json_t * j_user, void * cls);
+  int      (* user_middleware_module_get_profile)(struct config_module * config, const char * username, json_t * j_user, void * cls);
+  int      (* user_middleware_module_update)(struct config_module * config, const char * username, json_t * j_user, void * cls);
+  int      (* user_middleware_module_delete)(struct config_module * config, const char * username, json_t * j_user, void * cls);
+};
 
 /**
  * Structure used to store a user module instance
@@ -165,6 +195,16 @@ struct _user_module_instance {
   short int             multiple_passwords;
 };
 
+/**
+ * Structure used to store a user middleware module instance
+ */
+struct _user_middleware_module_instance {
+  char                           * name;
+  struct _user_middleware_module * module;
+  void                           * cls;
+  short int                        enabled;
+};
+
 /**
  * Structure used to store a client module
  */
@@ -231,6 +271,8 @@ struct _user_auth_scheme_module_instance {
   json_int_t                        guasmi_expiration;
   json_int_t                        guasmi_max_use;
   short int                         guasmi_allow_user_register;
+  short int                         guasmi_forbid_user_profile;
+  short int                         guasmi_forbid_user_reset_credential;
   void                            * cls;
   short int                         enabled;
 };
@@ -250,6 +292,7 @@ struct _plugin_module {
   int      (* plugin_module_unload)(struct config_plugin * config);
   json_t * (* plugin_module_init)(struct config_plugin * config, const char * name, json_t * j_parameters, void ** cls);
   int      (* plugin_module_close)(struct config_plugin * config, const char * name, void * cls);
+  int      (* plugin_user_revoke)(struct config_plugin * config, const char * username, void * cls);
 };
 
 /**
@@ -262,6 +305,27 @@ struct _plugin_module_instance {
   short int               enabled;
 };
 
+#define GLWD_METRICS_AUTH_USER_VALID          "glewlwyd_auth_user_valid"
+#define GLWD_METRICS_AUTH_USER_VALID_SCHEME   "glewlwyd_auth_user_valid_scheme"
+#define GLWD_METRICS_AUTH_USER_INVALID        "glewlwyd_auth_user_invalid"
+#define GLWD_METRICS_AUTH_USER_INVALID_SCHEME "glewlwyd_auth_user_invalid_scheme"
+#define GLWD_METRICS_DATABSE_ERROR            "glewlwyd_database_error"
+
+/**
+ * Structure used to store a prometheus metrics
+ */
+struct _glwd_metrics_data {
+  char * label;
+  size_t counter;
+};
+
+struct _glwd_metric {
+  char                      * name;
+  char                      * help;
+  struct _glwd_metrics_data * data;
+  size_t                      data_size;
+};
+
 /**
  * Structure used to store the global application config
  */
@@ -269,10 +333,12 @@ struct config_elements {
   char *                                         config_file;
   unsigned int                                   port;
   char *                                         bind_address;
+  char *                                         bind_address_metrics;
   char *                                         external_url;
   char *                                         api_prefix;
   char *                                         cookie_domain;
   unsigned int                                   cookie_secure;
+  unsigned int                                   add_x_frame_option_header_deny;
   unsigned short                                 log_mode_args;
   unsigned short                                 log_level_args;
   unsigned long                                  log_mode;
@@ -289,15 +355,21 @@ struct config_elements {
   struct _h_connection *                         conn;
   struct _u_instance *                           instance;
   unsigned int                                   instance_initialized;
+  struct _u_instance *                           instance_metrics;
+  unsigned int                                   instance_metrics_initialized;
   char *                                         session_key;
   unsigned int                                   session_expiration;
   unsigned int                                   salt_length;
   digest_algorithm                               hash_algorithm;
   char *                                         login_url;
   unsigned int                                   delete_profile;
+  pthread_mutex_t                                module_lock;
   char *                                         user_module_path;
   struct _pointer_list *                         user_module_list;
   struct _pointer_list *                         user_module_instance_list;
+  char *                                         user_middleware_module_path;
+  struct _pointer_list *                         user_middleware_module_list;
+  struct _pointer_list *                         user_middleware_module_instance_list;
   char *                                         client_module_path;
   struct _pointer_list *                         client_module_list;
   struct _pointer_list *                         client_module_instance_list;
@@ -309,6 +381,11 @@ struct config_elements {
   struct _pointer_list *                         plugin_module_instance_list;
   struct config_plugin *                         config_p;
   struct config_module *                         config_m;
+  unsigned short                                 metrics_endpoint;
+  unsigned int                                   metrics_endpoint_port;
+  unsigned short                                 metrics_endpoint_admin_session;
+  pthread_mutex_t                                metrics_lock;
+  struct _pointer_list                           metrics_list;
 };
 
 /**
@@ -319,17 +396,17 @@ struct config_plugin {
   struct config_elements * glewlwyd_config;
   int      (* glewlwyd_callback_add_plugin_endpoint)(struct config_plugin * config, const char * method, const char * name, const char * url, unsigned int priority, int (* callback)(const struct _u_request * request, struct _u_response * response, void * user_data), void * user_data);
   int      (* glewlwyd_callback_remove_plugin_endpoint)(struct config_plugin * config, const char * method, const char * name, const char * url);
-  
+
   // Session callback functions
   json_t * (* glewlwyd_callback_check_session_valid)(struct config_plugin * config, const struct _u_request * request, const char * scope_list);
   json_t * (* glewlwyd_callback_check_user_valid)(struct config_plugin * config, const char * username, const char * password, const char * scope_list);
   json_t * (* glewlwyd_callback_check_client_valid)(struct config_plugin * config, const char * client_id, const char * password);
   int      (* glewlwyd_callback_trigger_session_used)(struct config_plugin * config, const struct _u_request * request, const char * scope_list);
   time_t   (* glewlwyd_callback_get_session_age)(struct config_plugin * config, const struct _u_request * request, const char * scope_list);
-  
+
   // Client callback functions
   json_t * (* glewlwyd_callback_get_client_granted_scopes)(struct config_plugin * config, const char * client_id, const char * username, const char * scope_list);
-  
+
   // User CRUD
   json_t * (* glewlwyd_plugin_callback_get_user_list)(struct config_plugin * config, const char * pattern, size_t offset, size_t limit);
   json_t * (* glewlwyd_plugin_callback_get_user)(struct config_plugin * config, const char * username);
@@ -339,7 +416,7 @@ struct config_plugin {
   int      (* glewlwyd_plugin_callback_set_user)(struct config_plugin * config, const char * username, json_t * j_user);
   int      (* glewlwyd_plugin_callback_user_update_password)(struct config_plugin * config, const char * username, const char * password);
   int      (* glewlwyd_plugin_callback_delete_user)(struct config_plugin * config, const char * username);
-  
+
   // Client CRUD
   json_t * (* glewlwyd_plugin_callback_get_client_list)(struct config_plugin * config, const char * pattern, size_t offset, size_t limit);
   json_t * (* glewlwyd_plugin_callback_get_client)(struct config_plugin * config, const char * client_id);
@@ -349,12 +426,17 @@ struct config_plugin {
   int      (* glewlwyd_plugin_callback_delete_client)(struct config_plugin * config, const char * client_id);
 
   // Register scheme functions
+  json_t * (* glewlwyd_plugin_callback_get_scheme_module)(struct config_plugin * config, const char * mod_name);
   json_t * (* glewlwyd_plugin_callback_get_scheme_list)(struct config_plugin * config, const char * username);
   json_t * (* glewlwyd_plugin_callback_scheme_register)(struct config_plugin * config, const char * mod_name, const struct _u_request * http_request, const char * username, json_t * j_scheme_data);
   json_t * (* glewlwyd_plugin_callback_scheme_register_get)(struct config_plugin * config, const char * mod_name, const struct _u_request * http_request, const char * username);
   int      (* glewlwyd_plugin_callback_scheme_deregister)(struct config_plugin * config, const char * mod_name, const char * username);
   int      (* glewlwyd_plugin_callback_scheme_can_use)(struct config_plugin * config, const char * mod_name, const char * username);
   
+  // Prometheus metrics functions
+  int      (* glewlwyd_plugin_callback_metrics_add_metric)(struct config_plugin * config, const char * name, const char * help);
+  int      (* glewlwyd_plugin_callback_metrics_increment_counter)(struct config_plugin * config, const char * name, size_t inc, ...);
+
   // Misc functions
   char   * (* glewlwyd_callback_get_plugin_external_url)(struct config_plugin * config, const char * name);
   char   * (* glewlwyd_callback_get_login_url)(struct config_plugin * config, const char * client_id, const char * scope_list, const char * callback_url, struct _u_map * additional_parameters);
@@ -377,6 +459,8 @@ struct config_module {
   int                    (* glewlwyd_module_callback_set_user)(struct config_module * config, const char * username, json_t * j_user);
   int                    (* glewlwyd_module_callback_check_user_password)(struct config_module * config, const char * username, const char * password);
   json_t               * (* glewlwyd_module_callback_check_user_session)(struct config_module * config, const struct _u_request * request, const char * username);
+  int                    (* glewlwyd_module_callback_metrics_add_metric)(struct config_module * config, const char * name, const char * help);
+  int                    (* glewlwyd_module_callback_metrics_increment_counter)(struct config_module * config, const char * name, size_t inc, ...);
 };
 
 /**
@@ -390,11 +474,10 @@ char * rand_string_nonce(char * str, size_t str_size);
 char * rand_string_from_charset(char * str, size_t str_size, const char * charset);
 int rand_code(char * str, size_t str_size);
 char * join_json_string_array(json_t * j_array, const char * separator);
-char * url_encode(const char * str);
 int generate_digest(digest_algorithm digest, const char * data, int use_salt, char * out_digest);
 int generate_digest_raw(digest_algorithm digest, const unsigned char * data, size_t data_len, unsigned char * out_digest, size_t * out_digest_len);
 char * generate_hash(digest_algorithm digest, const char * data);
-int generate_digest_pbkdf2(const char * data, const char * salt, char * out_digest);
+int generate_digest_pbkdf2(const char * data, unsigned int iterations, const char * salt, char * out_digest);
 
 /**
  * Check if the result json object has a "result" element that is equal to value
@@ -424,6 +507,19 @@ int      user_module_delete(struct config_module * config, const char * username
 int      user_module_check_password(struct config_module * config, const char * username, const char * password, void * cls);
 int      user_module_update_password(struct config_module * config, const char * username, const char ** new_passwords, size_t new_passwords_len, void * cls);
 
+/**
+ * User middleware functions prototype
+ */
+json_t * user_middleware_module_load(struct config_module * config);
+int      user_middleware_module_unload(struct config_module * config);
+json_t * user_middleware_module_init(struct config_module * config, json_t * j_parameters, void ** cls);
+int      user_middleware_module_close(struct config_module * config, void * cls);
+int      user_middleware_module_get_list(struct config_module * config, json_t * j_user_list, void * cls);
+int      user_middleware_module_get(struct config_module * config, const char * username, json_t * j_user, void * cls);
+int      user_middleware_module_get_profile(struct config_module * config, const char * username, json_t * j_user, void * cls);
+int      user_middleware_module_update(struct config_module * config, const char * username, json_t * j_user, void * cls);
+int      user_middleware_module_delete(struct config_module * config, const char * username, json_t * j_user, void * cls);
+
 /**
  * Client functions prototypes
  */
