@@ -28,7 +28,7 @@
 #ifndef __GLEWLWYD_H_
 #define __GLEWLWYD_H_
 
-#define _GLEWLWYD_VERSION_ "2.5.2"
+#define _GLEWLWYD_VERSION_ "2.6.1"
 
 #include <jansson.h>
 
@@ -59,6 +59,7 @@
 
 // Configuration default values
 #define GLEWLWYD_DEFAULT_PORT                              4593
+#define GLEWLWYD_DEFAULT_METRICS_PORT                      4594
 #define GLEWLWYD_DEFAULT_API_PREFIX                        "api"
 #define GLEWLWYD_DEFAULT_ALLOW_ORIGIN                      "*"
 #define GLEWLWYD_DEFAULT_ADMIN_SCOPE                       "g_admin"
@@ -81,6 +82,7 @@
 
 // Data tables
 #define GLEWLWYD_TABLE_USER_MODULE_INSTANCE                    "g_user_module_instance"
+#define GLEWLWYD_TABLE_USER_MIDDLEWARE_MODULE_INSTANCE         "g_user_middleware_module_instance"
 #define GLEWLWYD_TABLE_USER_AUTH_SCHEME_MODULE_INSTANCE        "g_user_auth_scheme_module_instance"
 #define GLEWLWYD_TABLE_CLIENT_MODULE_INSTANCE                  "g_client_module_instance"
 #define GLEWLWYD_TABLE_PLUGIN_MODULE_INSTANCE                  "g_plugin_module_instance"
@@ -97,41 +99,47 @@
 #define GLEWLWYD_MODULE_ACTION_START 1
 
 // Environment variables names
-#define GLEWLWYD_ENV_PORT                       "GLWD_PORT"
-#define GLEWLWYD_ENV_BIND_ADDRESS               "GLWD_BIND_ADDRESS"
-#define GLEWLWYD_ENV_API_PREFIX                 "GLWD_API_PREFIX"
-#define GLEWLWYD_ENV_EXTERNAL_URL               "GLWD_EXTERNAL_URL"
-#define GLEWLWYD_ENV_LOGIN_URL                  "GLWD_LOGIN_URL"
-#define GLEWLWYD_ENV_PROFILE_DELETE             "GLWD_PROFILE_DELETE"
-#define GLEWLWYD_ENV_STATIC_FILES_PATH          "GLWD_STATIC_FILES_PATH"
-#define GLEWLWYD_ENV_STATIC_FILES_MIME_TYPES    "GLWD_STATIC_FILES_MIME_TYPES"
-#define GLEWLWYD_ENV_ALLOW_ORIGIN               "GLWD_ALLOW_ORIGIN"
-#define GLEWLWYD_ENV_LOG_MODE                   "GLWD_LOG_MODE"
-#define GLEWLWYD_ENV_LOG_LEVEL                  "GLWD_LOG_LEVEL"
-#define GLEWLWYD_ENV_LOG_FILE                   "GLWD_LOG_FILE"
-#define GLEWLWYD_ENV_COOKIE_DOMAIN              "GLWD_COOKIE_DOMAIN"
-#define GLEWLWYD_ENV_COOKIE_SECURE              "GLWD_COOKIE_SECURE"
-#define GLEWLWYD_ENV_SESSION_EXPIRATION         "GLWD_SESSION_EXPIRATION"
-#define GLEWLWYD_ENV_SESSION_KEY                "GLWD_SESSION_KEY"
-#define GLEWLWYD_ENV_ADMIN_SCOPE                "GLWD_ADMIN_SCOPE"
-#define GLEWLWYD_ENV_PROFILE_SCOPE              "GLWD_PROFILE_SCOPE"
-#define GLEWLWYD_ENV_USER_MODULE_PATH           "GLWD_USER_MODULE_PATH"
-#define GLEWLWYD_ENV_CLIENT_MODULE_PATH         "GLWD_CLIENT_MODULE_PATH"
-#define GLEWLWYD_ENV_AUTH_SCHEME_MODULE_PATH    "GLWD_AUTH_SCHEME_MODULE_PATH"
-#define GLEWLWYD_ENV_PLUGIN_MODULE_PATH         "GLWD_PLUGIN_MODULE_PATH"
-#define GLEWLWYD_ENV_USE_SECURE_CONNECTION      "GLWD_USE_SECURE_CONNECTION"
-#define GLEWLWYD_ENV_SECURE_CONNECTION_KEY_FILE "GLWD_SECURE_CONNECTION_KEY_FILE"
-#define GLEWLWYD_ENV_SECURE_CONNECTION_PEM_FILE "GLWD_SECURE_CONNECTION_PEM_FILE"
-#define GLEWLWYD_ENV_SECURE_CONNECTION_CA_FILE  "GLWD_SECURE_CONNECTION_CA_FILE"
-#define GLEWLWYD_ENV_HASH_ALGORITHM             "GLWD_HASH_ALGORITHM"
-#define GLEWLWYD_ENV_DATABASE_TYPE              "GLWD_DATABASE_TYPE"
-#define GLEWLWYD_ENV_DATABASE_MARIADB_HOST      "GLWD_DATABASE_MARIADB_HOST"
-#define GLEWLWYD_ENV_DATABASE_MARIADB_USER      "GLWD_DATABASE_MARIADB_USER"
-#define GLEWLWYD_ENV_DATABASE_MARIADB_PASSWORD  "GLWD_DATABASE_MARIADB_PASSWORD"
-#define GLEWLWYD_ENV_DATABASE_MARIADB_DBNAME    "GLWD_DATABASE_MARIADB_DBNAME"
-#define GLEWLWYD_ENV_DATABASE_MARIADB_PORT      "GLWD_DATABASE_MARIADB_PORT"
-#define GLEWLWYD_ENV_DATABASE_SQLITE3_PATH      "GLWD_DATABASE_SQLITE3_PATH"
-#define GLEWLWYD_ENV_DATABASE_POSTGRE_CONNINFO  "GLWD_DATABASE_POSTGRE_CONNINFO"
+#define GLEWLWYD_ENV_PORT                        "GLWD_PORT"
+#define GLEWLWYD_ENV_BIND_ADDRESS                "GLWD_BIND_ADDRESS"
+#define GLEWLWYD_ENV_API_PREFIX                  "GLWD_API_PREFIX"
+#define GLEWLWYD_ENV_EXTERNAL_URL                "GLWD_EXTERNAL_URL"
+#define GLEWLWYD_ENV_LOGIN_URL                   "GLWD_LOGIN_URL"
+#define GLEWLWYD_ENV_PROFILE_DELETE              "GLWD_PROFILE_DELETE"
+#define GLEWLWYD_ENV_STATIC_FILES_PATH           "GLWD_STATIC_FILES_PATH"
+#define GLEWLWYD_ENV_STATIC_FILES_MIME_TYPES     "GLWD_STATIC_FILES_MIME_TYPES"
+#define GLEWLWYD_ENV_ALLOW_ORIGIN                "GLWD_ALLOW_ORIGIN"
+#define GLEWLWYD_ENV_LOG_MODE                    "GLWD_LOG_MODE"
+#define GLEWLWYD_ENV_LOG_LEVEL                   "GLWD_LOG_LEVEL"
+#define GLEWLWYD_ENV_LOG_FILE                    "GLWD_LOG_FILE"
+#define GLEWLWYD_ENV_COOKIE_DOMAIN               "GLWD_COOKIE_DOMAIN"
+#define GLEWLWYD_ENV_COOKIE_SECURE               "GLWD_COOKIE_SECURE"
+#define GLEWLWYD_ENV_ADD_X_FRAME_DENY            "GLWD_ADD_X_FRAME_DENY"
+#define GLEWLWYD_ENV_SESSION_EXPIRATION          "GLWD_SESSION_EXPIRATION"
+#define GLEWLWYD_ENV_SESSION_KEY                 "GLWD_SESSION_KEY"
+#define GLEWLWYD_ENV_ADMIN_SCOPE                 "GLWD_ADMIN_SCOPE"
+#define GLEWLWYD_ENV_PROFILE_SCOPE               "GLWD_PROFILE_SCOPE"
+#define GLEWLWYD_ENV_USER_MODULE_PATH            "GLWD_USER_MODULE_PATH"
+#define GLEWLWYD_ENV_USER_MIDDLEWARE_MODULE_PATH "GLWD_USER_MIDDLEWARE_MODULE_PATH"
+#define GLEWLWYD_ENV_CLIENT_MODULE_PATH          "GLWD_CLIENT_MODULE_PATH"
+#define GLEWLWYD_ENV_AUTH_SCHEME_MODULE_PATH     "GLWD_AUTH_SCHEME_MODULE_PATH"
+#define GLEWLWYD_ENV_PLUGIN_MODULE_PATH          "GLWD_PLUGIN_MODULE_PATH"
+#define GLEWLWYD_ENV_USE_SECURE_CONNECTION       "GLWD_USE_SECURE_CONNECTION"
+#define GLEWLWYD_ENV_SECURE_CONNECTION_KEY_FILE  "GLWD_SECURE_CONNECTION_KEY_FILE"
+#define GLEWLWYD_ENV_SECURE_CONNECTION_PEM_FILE  "GLWD_SECURE_CONNECTION_PEM_FILE"
+#define GLEWLWYD_ENV_SECURE_CONNECTION_CA_FILE   "GLWD_SECURE_CONNECTION_CA_FILE"
+#define GLEWLWYD_ENV_HASH_ALGORITHM              "GLWD_HASH_ALGORITHM"
+#define GLEWLWYD_ENV_DATABASE_TYPE               "GLWD_DATABASE_TYPE"
+#define GLEWLWYD_ENV_DATABASE_MARIADB_HOST       "GLWD_DATABASE_MARIADB_HOST"
+#define GLEWLWYD_ENV_DATABASE_MARIADB_USER       "GLWD_DATABASE_MARIADB_USER"
+#define GLEWLWYD_ENV_DATABASE_MARIADB_PASSWORD   "GLWD_DATABASE_MARIADB_PASSWORD"
+#define GLEWLWYD_ENV_DATABASE_MARIADB_DBNAME     "GLWD_DATABASE_MARIADB_DBNAME"
+#define GLEWLWYD_ENV_DATABASE_MARIADB_PORT       "GLWD_DATABASE_MARIADB_PORT"
+#define GLEWLWYD_ENV_DATABASE_SQLITE3_PATH       "GLWD_DATABASE_SQLITE3_PATH"
+#define GLEWLWYD_ENV_DATABASE_POSTGRE_CONNINFO   "GLWD_DATABASE_POSTGRE_CONNINFO"
+#define GLEWLWYD_ENV_METRICS                     "GLWD_METRICS"
+#define GLEWLWYD_ENV_METRICS_PORT                "GLWD_METRICS_PORT"
+#define GLEWLWYD_ENV_METRICS_ADMIN               "GLWD_METRICS_ADMIN"
+#define GLEWLWYD_ENV_METRICS_BIND_ADDRESS        "GLWD_METRICS_BIND_ADDRESS"
 
 // Main functions and misc functions
 int build_config_from_env(struct config_elements * config);
@@ -144,6 +152,8 @@ void print_help(FILE * output);
 char * get_file_content(const char * file_path);
 int    load_user_module_instance_list(struct config_elements * config);
 int    init_user_module_list(struct config_elements * config);
+int    load_user_middleware_module_instance_list(struct config_elements * config);
+int    init_user_middleware_module_list(struct config_elements * config);
 int    load_user_auth_scheme_module_instance_list(struct config_elements * config);
 int    init_user_auth_scheme_module_list(struct config_elements * config);
 int    init_client_module_list(struct config_elements * config);
@@ -154,6 +164,8 @@ struct _client_module_instance * get_client_module_instance(struct config_elemen
 struct _client_module * get_client_module_lib(struct config_elements * config, const char * name);
 struct _user_module_instance * get_user_module_instance(struct config_elements * config, const char * name);
 struct _user_module * get_user_module_lib(struct config_elements * config, const char * name);
+struct _user_middleware_module_instance * get_user_middleware_module_instance(struct config_elements * config, const char * name);
+struct _user_middleware_module * get_user_middleware_module_lib(struct config_elements * config, const char * name);
 struct _user_auth_scheme_module_instance * get_user_auth_scheme_module_instance(struct config_elements * config, const char * name);
 struct _user_auth_scheme_module * get_user_auth_scheme_module_lib(struct config_elements * config, const char * name);
 struct _plugin_module_instance * get_plugin_module_instance(struct config_elements * config, const char * name);
@@ -205,6 +217,7 @@ json_t * get_validated_auth_scheme_list_from_scope_list(struct config_elements *
 int is_scope_list_valid_for_session(struct config_elements * config, const char * scope_list, const char * session_uid);
 json_t * get_client_user_scope_grant(struct config_elements * config, const char * client_id, const char * username, const char * scope_list);
 json_t * get_granted_scopes_for_client(struct config_elements * config, json_t * j_user, const char * client_id, const char * scope_list);
+json_t * get_client_grant_list(struct config_elements * config, const char * username, size_t offset, size_t limit);
 int set_granted_scopes_for_client(struct config_elements * config, json_t * j_user, const char * client_id, const char * scope_list);
 json_t * get_scope_list_allowed_for_session(struct config_elements * config, const char * scope_list, const char * session_uid);
 
@@ -222,6 +235,17 @@ json_t * manage_user_module(struct config_elements * config, const char * name,
 void close_user_module_instance_list(struct config_elements * config);
 void close_user_module_list(struct config_elements * config);
 
+// User middleware module functions
+json_t * get_user_middleware_module_list(struct config_elements * config);
+json_t * get_user_middleware_module(struct config_elements * config, const char * name);
+json_t * is_user_middleware_module_valid(struct config_elements * config, json_t * j_module, int add);
+json_t * add_user_middleware_module(struct config_elements * config, json_t * j_module);
+int set_user_middleware_module(struct config_elements * config, const char * name, json_t * j_module);
+int delete_user_middleware_module(struct config_elements * config, const char * name);
+json_t * manage_user_middleware_module(struct config_elements * config, const char * name, int action);
+void close_user_middleware_module_instance_list(struct config_elements * config);
+void close_user_middleware_module_list(struct config_elements * config);
+
 // User auth scheme module functions
 json_t * get_user_auth_scheme_module_list(struct config_elements * config);
 json_t * get_user_auth_scheme_module(struct config_elements * config, const char * name);
@@ -282,11 +306,14 @@ json_t * glewlwyd_plugin_callback_is_client_valid(struct config_plugin * config,
 int glewlwyd_plugin_callback_add_client(struct config_plugin * config, json_t * j_client);
 int glewlwyd_plugin_callback_set_client(struct config_plugin * config, const char * client_id, json_t * j_client);
 int glewlwyd_plugin_callback_delete_client(struct config_plugin * config, const char * client_id);
+json_t * glewlwyd_plugin_callback_get_scheme_module(struct config_plugin * config, const char * mod_name);
 json_t * glewlwyd_plugin_callback_get_scheme_list(struct config_plugin * config, const char * username);
 json_t * glewlwyd_plugin_callback_scheme_register(struct config_plugin * config, const char * mod_name, const struct _u_request * http_request, const char * username, json_t * j_scheme_data);
 json_t * glewlwyd_plugin_callback_scheme_register_get(struct config_plugin * config, const char * mod_name, const struct _u_request * http_request, const char * username);
 int glewlwyd_plugin_callback_scheme_can_use(struct config_plugin * config, const char * mod_name, const char * username);
 int glewlwyd_plugin_callback_scheme_deregister(struct config_plugin * config, const char * mod_name, const char * username);
+int glewlwyd_plugin_callback_metrics_add_metric(struct config_plugin * config, const char * name, const char * help);
+int glewlwyd_plugin_callback_metrics_increment_counter(struct config_plugin * config, const char * name, size_t inc, ...);
 
 // User CRUD functions
 json_t * get_user_list(struct config_elements * config, const char * pattern, size_t offset, size_t limit, const char * source);
@@ -300,6 +327,9 @@ json_t * glewlwyd_module_callback_get_user(struct config_module * config, const
 int glewlwyd_module_callback_set_user(struct config_module * config, const char * username, json_t * j_user);
 int glewlwyd_module_callback_check_user_password(struct config_module * config, const char * username, const char * password);
 json_t * glewlwyd_module_callback_check_user_session(struct config_module * config, const struct _u_request * request, const char * username);
+int glewlwyd_module_metrics_increment_counter(struct config_module * config, const char * metrics_name, size_t inc, const char * module_type, const char * module_name);
+int glewlwyd_module_callback_metrics_add_metric(struct config_module * config, const char * name, const char * help);
+int glewlwyd_module_callback_metrics_increment_counter(struct config_module * config, const char * name, size_t inc, ...);
 
 // Client CRUD functions
 json_t * get_client_list(struct config_elements * config, const char * pattern, size_t offset, size_t limit, const char * source);
@@ -323,12 +353,20 @@ json_t * get_api_key_list(struct config_elements * config, const char * pattern,
 json_t * generate_api_key(struct config_elements * config, const char * username, const char * issued_for, const char * user_agent);
 int disable_api_key(struct config_elements * config, const char * token_hash);
 
+// Metrics functions
+void glewlwyd_metrics_close(struct config_elements * config);
+int glewlwyd_metrics_init(struct config_elements * config);
+void free_glwd_metrics(void * data);
+int glewlwyd_metrics_add_metric(struct config_elements * config, const char * name, const char * help);
+int glewlwyd_metrics_increment_counter_va(struct config_elements * config, const char * name, size_t inc, ...);
+int glewlwyd_metrics_increment_counter(struct config_elements * config, const char * name, const char * label, size_t inc);
+char * glewlwyd_metrics_build_label(va_list vl_label);
+
 // Callback functions
 int callback_glewlwyd_check_user_session (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_check_admin_session (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_check_admin_session_or_api_key (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_check_admin_session_delegate (const struct _u_request * request, struct _u_response * response, void * user_data);
-int callback_glewlwyd_close_check_session (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_check_user_profile_valid (const struct _u_request * request, struct _u_response * response, void * user_data);
 
 int callback_glewlwyd_user_auth (const struct _u_request * request, struct _u_response * response, void * user_data);
@@ -341,11 +379,13 @@ int callback_glewlwyd_user_update_profile (const struct _u_request * request, st
 int callback_glewlwyd_user_delete_profile (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_user_update_password (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_user_get_plugin_list (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_user_get_client_grant_list (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_user_get_session_list (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_user_get_scheme_list (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_delete_session (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_user_auth_register (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_user_auth_register_get (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_scheme_check_forbid_profile (const struct _u_request * request, struct _u_response * response, void * user_data);
 
 int callback_glewlwyd_user_auth_register_delegate (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_user_auth_register_get_delegate (const struct _u_request * request, struct _u_response * response, void * user_data);
@@ -366,49 +406,59 @@ int callback_glewlwyd_set_user_module (const struct _u_request * request, struct
 int callback_glewlwyd_delete_user_module (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_glewlwyd_manage_user_module (const struct _u_request * request, struct _u_response * response, void * user_data);
 
-int callback_glewlwyd_get_user_auth_scheme_module_list (const struct _u_request * request, struct _u_response * response, void * user_auth_scheme_data);
-int callback_glewlwyd_get_user_auth_scheme_module (const struct _u_request * request, struct _u_response * response, void * user_auth_scheme_data);
-int callback_glewlwyd_add_user_auth_scheme_module (const struct _u_request * request, struct _u_response * response, void * user_auth_scheme_data);
-int callback_glewlwyd_set_user_auth_scheme_module (const struct _u_request * request, struct _u_response * response, void * user_auth_scheme_data);
-int callback_glewlwyd_delete_user_auth_scheme_module (const struct _u_request * request, struct _u_response * response, void * user_auth_scheme_data);
-int callback_glewlwyd_manage_user_auth_scheme_module (const struct _u_request * request, struct _u_response * response, void * user_auth_scheme_data);
+int callback_glewlwyd_get_user_middleware_module_list (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_get_user_middleware_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_add_user_middleware_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_set_user_middleware_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_delete_user_middleware_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_manage_user_middleware_module (const struct _u_request * request, struct _u_response * response, void * user_data);
 
-int callback_glewlwyd_get_client_module_list (const struct _u_request * request, struct _u_response * response, void * client_data);
-int callback_glewlwyd_get_client_module (const struct _u_request * request, struct _u_response * response, void * client_data);
-int callback_glewlwyd_add_client_module (const struct _u_request * request, struct _u_response * response, void * client_data);
-int callback_glewlwyd_set_client_module (const struct _u_request * request, struct _u_response * response, void * client_data);
-int callback_glewlwyd_delete_client_module (const struct _u_request * request, struct _u_response * response, void * client_data);
-int callback_glewlwyd_manage_client_module (const struct _u_request * request, struct _u_response * response, void * client_data);
+int callback_glewlwyd_get_user_auth_scheme_module_list (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_get_user_auth_scheme_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_add_user_auth_scheme_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_set_user_auth_scheme_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_delete_user_auth_scheme_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_manage_user_auth_scheme_module (const struct _u_request * request, struct _u_response * response, void * user_data);
 
-int callback_glewlwyd_get_plugin_module_list (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_get_plugin_module (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_add_plugin_module (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_set_plugin_module (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_delete_plugin_module (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_manage_plugin_module (const struct _u_request * request, struct _u_response * response, void * plugin_data);
+int callback_glewlwyd_get_client_module_list (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_get_client_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_add_client_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_set_client_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_delete_client_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_manage_client_module (const struct _u_request * request, struct _u_response * response, void * user_data);
 
-int callback_glewlwyd_get_user_list (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_get_user (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_add_user (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_set_user (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_delete_user (const struct _u_request * request, struct _u_response * response, void * plugin_data);
+int callback_glewlwyd_get_plugin_module_list (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_get_plugin_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_add_plugin_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_set_plugin_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_delete_plugin_module (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_manage_plugin_module (const struct _u_request * request, struct _u_response * response, void * user_data);
 
-int callback_glewlwyd_get_client_list (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_get_client (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_add_client (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_set_client (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_delete_client (const struct _u_request * request, struct _u_response * response, void * plugin_data);
+int callback_glewlwyd_get_user_list (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_get_user (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_add_user (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_set_user (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_delete_user (const struct _u_request * request, struct _u_response * response, void * user_data);
 
-int callback_glewlwyd_get_scope_list (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_get_scope (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_add_scope (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_set_scope (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_delete_scope (const struct _u_request * request, struct _u_response * response, void * plugin_data);
+int callback_glewlwyd_get_client_list (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_get_client (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_add_client (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_set_client (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_delete_client (const struct _u_request * request, struct _u_response * response, void * user_data);
 
-int callback_glewlwyd_get_api_key_list (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_delete_api_key (const struct _u_request * request, struct _u_response * response, void * plugin_data);
-int callback_glewlwyd_add_api_key (const struct _u_request * request, struct _u_response * response, void * plugin_data);
+int callback_glewlwyd_get_scope_list (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_get_scope (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_add_scope (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_set_scope (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_delete_scope (const struct _u_request * request, struct _u_response * response, void * user_data);
+
+int callback_glewlwyd_get_api_key_list (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_delete_api_key (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_glewlwyd_add_api_key (const struct _u_request * request, struct _u_response * response, void * user_data);
+
+int callback_metrics (const struct _u_request * request, struct _u_response * response, void * user_data);
 
 int callback_default (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_404_if_necessary (const struct _u_request * request, struct _u_response * response, void * user_data);
 
 #endif
