@@ -286,6 +286,7 @@ int glewlwyd_callback_trigger_session_used(struct config_plugin * config, const
             json_decref(j_query);
             if (res != H_OK) {
               y_log_message(Y_LOG_LEVEL_ERROR, "glewlwyd_callback_trigger_session_used - Error h_update for password scheme");
+              glewlwyd_metrics_increment_counter_va(config->glewlwyd_config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
               ret = G_ERROR_DB;
             }
           }
@@ -329,6 +330,7 @@ int glewlwyd_callback_trigger_session_used(struct config_plugin * config, const
                 json_decref(j_query);
                 if (res != H_OK) {
                   y_log_message(Y_LOG_LEVEL_ERROR, "glewlwyd_callback_trigger_session_used - Error h_update for scheme %s/%s", json_string_value(json_object_get(j_scheme, "scheme_type")), json_string_value(json_object_get(j_scheme, "scheme_name")));
+                  glewlwyd_metrics_increment_counter_va(config->glewlwyd_config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                   ret = G_ERROR_DB;
                 }
               }
@@ -407,13 +409,13 @@ char * glewlwyd_callback_get_login_url(struct config_plugin * config, const char
   int i;
   
   if (callback_url != NULL) {
-    encoded_callback_url = url_encode(callback_url);
+    encoded_callback_url = ulfius_url_encode(callback_url);
   }
   if (client_id != NULL) {
-    encoded_client_id = url_encode(client_id);
+    encoded_client_id = ulfius_url_encode(client_id);
   }
   if (scope_list != NULL) {
-    encoded_scope_list = url_encode(scope_list);
+    encoded_scope_list = ulfius_url_encode(scope_list);
   }
   if (additional_parameters != NULL) {
     keys = u_map_enum_keys(additional_parameters);
@@ -421,7 +423,7 @@ char * glewlwyd_callback_get_login_url(struct config_plugin * config, const char
       if (u_map_get(additional_parameters, keys[i]) == NULL) {
         query_additional_parameters = mstrcatf(query_additional_parameters, "&%s", keys[i]);
       } else {
-        value_encoded = url_encode(u_map_get(additional_parameters, keys[i]));
+        value_encoded = ulfius_url_encode(u_map_get(additional_parameters, keys[i]));
         query_additional_parameters = mstrcatf(query_additional_parameters, "&%s=%s", keys[i], value_encoded);
         o_free(value_encoded);
       }
@@ -546,6 +548,14 @@ int glewlwyd_plugin_callback_delete_client(struct config_plugin * config, const
   return delete_client(config->glewlwyd_config, client_id, NULL);
 }
 
+json_t * glewlwyd_plugin_callback_get_scheme_module(struct config_plugin * config, const char * mod_name) {
+  if (o_strlen(mod_name)) {
+    return get_user_auth_scheme_module(config->glewlwyd_config, mod_name);
+  } else {
+    return json_pack("{si}", "result", G_ERROR_PARAM);
+  }
+}
+
 json_t * glewlwyd_plugin_callback_get_scheme_list(struct config_plugin * config, const char * username) {
   if (o_strlen(username)) {
     return get_scheme_list_for_user(config->glewlwyd_config, username);
@@ -597,3 +607,30 @@ int glewlwyd_plugin_callback_scheme_deregister(struct config_plugin * config, co
   }
   return ret;
 }
+
+int glewlwyd_plugin_callback_metrics_add_metric(struct config_plugin * config, const char * name, const char * help) {
+  if (config != NULL) {
+    return glewlwyd_metrics_add_metric(config->glewlwyd_config, name, help);
+  } else {
+    return G_ERROR_PARAM;
+  }
+}
+
+int glewlwyd_plugin_callback_metrics_increment_counter(struct config_plugin * config, const char * name, size_t inc, ...) {
+  va_list vl;
+  char * label = NULL;
+  int ret = G_OK;
+
+  if (config != NULL && o_strlen(name)) {
+    va_start(vl, inc);
+    label = glewlwyd_metrics_build_label(vl);
+    va_end(vl);
+    
+    ret = glewlwyd_metrics_increment_counter(config->glewlwyd_config, name, label, inc);
+    o_free(label);
+  } else {
+    y_log_message(Y_LOG_LEVEL_ERROR, "glewlwyd_plugin_callback_metrics_increment_counter - Error input values");
+    ret = G_ERROR_PARAM;
+  }
+  return ret;
+}
