@@ -1,3 +1,6 @@
+DROP TABLE IF EXISTS gpo_ciba_scope;
+DROP TABLE IF EXISTS gpo_ciba_scheme;
+DROP TABLE IF EXISTS gpo_ciba;
 DROP TABLE IF EXISTS gpo_par_scope;
 DROP TABLE IF EXISTS gpo_par;
 DROP TABLE IF EXISTS gpo_rar;
@@ -29,6 +32,8 @@ CREATE TABLE gpo_code (
   gpoc_nonce VARCHAR(512),
   gpoc_claims_request TEXT DEFAULT NULL,
   gpoc_authorization_details TEXT DEFAULT NULL,
+  gpoc_s_hash VARCHAR(512),
+  gpoc_sid VARCHAR(128),
   gpoc_expires_at TIMESTAMPTZ NOT NULL DEFAULT NOW(),
   gpoc_issued_for VARCHAR(256), -- IP address or hostname
   gpoc_user_agent VARCHAR(256),
@@ -116,6 +121,8 @@ CREATE TABLE gpo_access_token_scope (
 -- Id token table, to store meta information on id token sent
 CREATE TABLE gpo_id_token (
   gpoi_id SERIAL PRIMARY KEY,
+  gpoc_id INTEGER,
+  gpor_id INTEGER,
   gpoi_plugin_name VARCHAR(256) NOT NULL,
   gpoi_authorization_type SMALLINT NOT NULL,
   gpoi_username VARCHAR(256),
@@ -124,7 +131,10 @@ CREATE TABLE gpo_id_token (
   gpoi_issued_for VARCHAR(256), -- IP address or hostname
   gpoi_user_agent VARCHAR(256),
   gpoi_hash VARCHAR(512),
-  gpoi_enabled SMALLINT DEFAULT 1
+  gpoi_sid VARCHAR(128),
+  gpoi_enabled SMALLINT DEFAULT 1,
+  FOREIGN KEY(gpoc_id) REFERENCES gpo_code(gpoc_id) ON DELETE CASCADE,
+  FOREIGN KEY(gpor_id) REFERENCES gpo_refresh_token(gpor_id) ON DELETE CASCADE
 );
 CREATE INDEX i_gpoi_hash ON gpo_id_token(gpoi_hash);
 
@@ -175,6 +185,7 @@ CREATE TABLE gpo_device_authorization (
   gpoda_issued_for VARCHAR(256), -- IP address or hostname of the deice client
   gpoda_device_code_hash VARCHAR(512) NOT NULL,
   gpoda_user_code_hash VARCHAR(512) NOT NULL,
+  gpoda_sid VARCHAR(128),
   gpoda_status SMALLINT DEFAULT 0, -- 0: created, 1: user verified, 2 device completed, 3 disabled
   gpoda_authorization_details TEXT DEFAULT NULL,
   gpoda_last_check TIMESTAMPTZ DEFAULT NOW()
@@ -243,6 +254,7 @@ CREATE TABLE gpo_par (
   gpop_issued_for VARCHAR(256), -- IP address or hostname
   gpop_user_agent VARCHAR(256)
 );
+CREATE INDEX i_gpop_client_id ON gpo_par(gpop_client_id);
 CREATE INDEX i_gpop_request_uri_hash ON gpo_par(gpop_request_uri_hash);
 CREATE INDEX i_gpop_code_challenge ON gpo_par(gpop_code_challenge);
 
@@ -252,3 +264,42 @@ CREATE TABLE gpo_par_scope (
   gpops_scope VARCHAR(128) NOT NULL,
   FOREIGN KEY(gpop_id) REFERENCES gpo_par(gpop_id) ON DELETE CASCADE
 );
+
+CREATE TABLE gpo_ciba (
+  gpob_id SERIAL PRIMARY KEY,
+  gpob_plugin_name VARCHAR(256) NOT NULL,
+  gpob_client_id VARCHAR(256) NOT NULL,
+  gpob_x5t_s256 VARCHAR(64),
+  gpob_username VARCHAR(256) NOT NULL,
+  gpob_client_notification_token VARCHAR(1024),
+  gpob_jti_hash VARCHAR(512),
+  gpob_auth_req_id VARCHAR(128),
+  gpob_user_req_id VARCHAR(128),
+  gpob_binding_message VARCHAR(256),
+  gpob_sid VARCHAR(128),
+  gpob_status SMALLINT DEFAULT 0, -- 0: created, 1: accepted, 2: error, 3: closed
+  gpob_expires_at TIMESTAMPTZ NOT NULL DEFAULT CURRENT_TIMESTAMP,
+  gpob_issued_for VARCHAR(256), -- IP address or hostname
+  gpob_user_agent VARCHAR(256),
+  gpob_enabled SMALLINT DEFAULT 1
+);
+CREATE INDEX i_gpob_client_id ON gpo_ciba(gpob_client_id);
+CREATE INDEX i_gpob_jti_hash ON gpo_ciba(gpob_jti_hash);
+CREATE INDEX i_gpob_client_notification_token ON gpo_ciba(gpob_client_notification_token);
+CREATE INDEX i_gpob_auth_req_id ON gpo_ciba(gpob_auth_req_id);
+CREATE INDEX i_gpob_user_req_id ON gpo_ciba(gpob_user_req_id);
+
+CREATE TABLE gpo_ciba_scope (
+  gpocs_id SERIAL PRIMARY KEY,
+  gpob_id INTEGER,
+  gpops_scope VARCHAR(128) NOT NULL,
+  gpobs_granted SMALLINT DEFAULT 0,
+  FOREIGN KEY(gpob_id) REFERENCES gpo_ciba(gpob_id) ON DELETE CASCADE
+);
+
+CREATE TABLE gpo_ciba_scheme (
+  gpobh_id SERIAL PRIMARY KEY,
+  gpob_id INTEGER,
+  gpobh_scheme_module VARCHAR(128) NOT NULL,
+  FOREIGN KEY(gpob_id) REFERENCES gpo_ciba(gpob_id) ON DELETE CASCADE
+);
