@@ -1,3 +1,6 @@
+DROP TABLE IF EXISTS gpo_ciba_scope;
+DROP TABLE IF EXISTS gpo_ciba_scheme;
+DROP TABLE IF EXISTS gpo_ciba;
 DROP TABLE IF EXISTS gpo_par_scope;
 DROP TABLE IF EXISTS gpo_par;
 DROP TABLE IF EXISTS gpo_rar;
@@ -29,6 +32,8 @@ CREATE TABLE gpo_code (
   gpoc_resource TEXT,
   gpoc_claims_request TEXT DEFAULT NULL,
   gpoc_authorization_details TEXT DEFAULT NULL,
+  gpoc_s_hash TEXT,
+  gpoc_sid TEXT,
   gpoc_expires_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
   gpoc_issued_for TEXT, -- IP address or hostname
   gpoc_user_agent TEXT,
@@ -108,7 +113,7 @@ CREATE INDEX i_gpoa_jti ON gpo_access_token(gpoa_jti);
 
 CREATE TABLE gpo_access_token_scope (
   gpoas_id INTEGER PRIMARY KEY AUTOINCREMENT,
-  gpoa_id INT(11),
+  gpoa_id INTEGER,
   gpoas_scope TEXT NOT NULL,
   FOREIGN KEY(gpoa_id) REFERENCES gpo_access_token(gpoa_id) ON DELETE CASCADE
 );
@@ -116,6 +121,8 @@ CREATE TABLE gpo_access_token_scope (
 -- Id token table, to store meta information on id token sent
 CREATE TABLE gpo_id_token (
   gpoi_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gpoc_id INTEGER,
+  gpor_id INTEGER,
   gpoi_plugin_name TEXT NOT NULL,
   gpoi_authorization_type INTEGER NOT NULL,
   gpoi_username TEXT,
@@ -124,7 +131,10 @@ CREATE TABLE gpo_id_token (
   gpoi_issued_for TEXT, -- IP address or hostname
   gpoi_user_agent TEXT,
   gpoi_hash TEXT,
-  gpoi_enabled INTEGER DEFAULT 1
+  gpoi_sid TEXT,
+  gpoi_enabled INTEGER DEFAULT 1,
+  FOREIGN KEY(gpoc_id) REFERENCES gpo_code(gpoc_id) ON DELETE CASCADE,
+  FOREIGN KEY(gpor_id) REFERENCES gpo_refresh_token(gpor_id) ON DELETE CASCADE
 );
 CREATE INDEX i_gpoi_hash ON gpo_id_token(gpoi_hash);
 
@@ -175,6 +185,7 @@ CREATE TABLE gpo_device_authorization (
   gpoda_issued_for TEXT, -- IP address or hostname of the deice client
   gpoda_device_code_hash TEXT NOT NULL,
   gpoda_user_code_hash TEXT NOT NULL,
+  gpoda_sid TEXT,
   gpoda_status INTEGER DEFAULT 0, -- 0: created, 1: user verified, 2 device completed, 3 disabled
   gpoda_authorization_details TEXT DEFAULT NULL,
   gpoda_last_check TIMESTAMP DEFAULT CURRENT_TIMESTAMP
@@ -243,6 +254,7 @@ CREATE TABLE gpo_par (
   gpop_issued_for TEXT, -- IP address or hostname
   gpop_user_agent TEXT
 );
+CREATE INDEX i_gpop_client_id ON gpo_par(gpop_client_id);
 CREATE INDEX i_gpop_request_uri_hash ON gpo_par(gpop_request_uri_hash);
 CREATE INDEX i_gpop_code_challenge ON gpo_par(gpop_code_challenge);
 
@@ -252,3 +264,42 @@ CREATE TABLE gpo_par_scope (
   gpops_scope TEXT NOT NULL,
   FOREIGN KEY(gpop_id) REFERENCES gpo_par(gpop_id) ON DELETE CASCADE
 );
+
+CREATE TABLE gpo_ciba (
+  gpob_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gpob_plugin_name TEXT NOT NULL,
+  gpob_client_id TEXT NOT NULL,
+  gpob_x5t_s256 TEXT,
+  gpob_username TEXT NOT NULL,
+  gpob_client_notification_token TEXT,
+  gpob_jti_hash TEXT,
+  gpob_auth_req_id TEXT,
+  gpob_user_req_id TEXT,
+  gpob_binding_message TEXT,
+  gpob_sid TEXT,
+  gpob_status INTEGER DEFAULT 0, -- 0: created, 1: accepted, 2: error, 3: closed
+  gpob_expires_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
+  gpob_issued_for TEXT, -- IP address or hostname
+  gpob_user_agent TEXT,
+  gpob_enabled INTEGER DEFAULT 1
+);
+CREATE INDEX i_gpob_client_id ON gpo_ciba(gpob_client_id);
+CREATE INDEX i_gpob_jti_hash ON gpo_ciba(gpob_jti_hash);
+CREATE INDEX i_gpob_client_notification_token ON gpo_ciba(gpob_client_notification_token);
+CREATE INDEX i_gpob_auth_req_id ON gpo_ciba(gpob_auth_req_id);
+CREATE INDEX i_gpob_user_req_id ON gpo_ciba(gpob_user_req_id);
+
+CREATE TABLE gpo_ciba_scope (
+  gpocs_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gpob_id INTEGER,
+  gpops_scope TEXT NOT NULL,
+  gpobs_granted INTEGER DEFAULT 0,
+  FOREIGN KEY(gpob_id) REFERENCES gpo_ciba(gpob_id) ON DELETE CASCADE
+);
+
+CREATE TABLE gpo_ciba_scheme (
+  gpobh_id INTEGER PRIMARY KEY AUTOINCREMENT,
+  gpob_id INTEGER,
+  gpobh_scheme_module TEXT NOT NULL,
+  FOREIGN KEY(gpob_id) REFERENCES gpo_ciba(gpob_id) ON DELETE CASCADE
+);
