@@ -33,7 +33,7 @@
 #include <jansson.h>
 #include <yder.h>
 #include <orcania.h>
-#include "../glewlwyd-common.h"
+#include "glewlwyd-common.h"
 
 #define GLEWLWYD_SCHEME_CERTIFICATE_TABLE_USER_CERTIFICATE "gs_user_certificate"
 
@@ -254,6 +254,7 @@ static int update_user_certificate_enabled_scheme_storage(struct config_module *
     ret = G_OK;
   } else {
     y_log_message(Y_LOG_LEVEL_DEBUG, "toggle_enabled_user_certificate_scheme_storage - Error executing j_query");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     ret = G_ERROR_DB;
   }
   return ret;
@@ -294,6 +295,7 @@ static int update_user_certificate_last_used_scheme_storage(struct config_module
     ret = G_OK;
   } else {
     y_log_message(Y_LOG_LEVEL_DEBUG, "toggle_enabled_user_certificate_scheme_storage - Error executing j_query");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     ret = G_ERROR_DB;
   }
   return ret;
@@ -319,6 +321,7 @@ static int delete_user_certificate_scheme_storage(struct config_module * config,
     ret = G_OK;
   } else {
     y_log_message(Y_LOG_LEVEL_DEBUG, "delete_user_certificate_scheme_storage - Error executing j_query");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     ret = G_ERROR_DB;
   }
   return ret;
@@ -470,6 +473,7 @@ static json_t * get_user_certificate_from_id_scheme_storage(struct config_module
     json_decref(j_result);
   } else {
     y_log_message(Y_LOG_LEVEL_DEBUG, "get_user_certificate_from_id_scheme_storage - Error executing j_query");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     j_return = json_pack("{si}", "result", G_ERROR_DB);
   }
   return j_return;
@@ -517,6 +521,7 @@ static json_t * get_user_certificate_list_scheme_storage(struct config_module *
     json_decref(j_result);
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "get_user_certificate_list - Error executing j_query");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     j_return = json_pack("{si}", "result", G_ERROR_DB);
   }
   return j_return;
@@ -591,6 +596,7 @@ static int add_user_certificate_scheme_storage(struct config_module * config, js
           ret = G_OK;
         } else {
           y_log_message(Y_LOG_LEVEL_ERROR, "add_user_certificate_scheme_storage - Error executing j_query");
+          config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
           ret = G_ERROR_DB;
         }
       } else if (check_result_value(j_result, G_OK)) {
@@ -726,7 +732,11 @@ static int is_user_certificate_valid_user_property(struct config_module * config
   
   if (check_result_value(j_user_list, G_OK)) {
     if (json_string_length(json_object_get(j_user_list, "dn"))) {
+#if GNUTLS_VERSION_NUMBER >= 0x030702
+      if (gnutls_x509_crt_get_dn3(cert, &cert_dn, 0) == GNUTLS_E_SUCCESS) {
+#else
       if (gnutls_x509_crt_get_dn2(cert, &cert_dn) == GNUTLS_E_SUCCESS) {
+#endif
         if (cert_dn.size == json_string_length(json_object_get(j_user_list, "dn")) && 
             0 == o_strncasecmp(json_string_value(json_object_get(j_user_list, "dn")), (const char *)cert_dn.data, cert_dn.size)) {
           ret = G_OK;
@@ -735,7 +745,7 @@ static int is_user_certificate_valid_user_property(struct config_module * config
         }
         gnutls_free(cert_dn.data);
       } else {
-        y_log_message(Y_LOG_LEVEL_ERROR, "is_user_certificate_valid_user_property - Error gnutls_x509_crt_get_dn2");
+        y_log_message(Y_LOG_LEVEL_ERROR, "is_user_certificate_valid_user_property - Error gnutls_x509_crt_get_dn3");
         ret = G_ERROR;
       }
     } else {
@@ -1702,6 +1712,7 @@ json_t * user_auth_scheme_module_identify(struct config_module * config, const s
             j_return = json_pack("{si}", "result", G_ERROR);
           }
         } else {
+          y_log_message(Y_LOG_LEVEL_DEBUG, "user_auth_scheme_module_identify certificate - use-scheme-storage isn't set");
           j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
         }
       } else {
@@ -1719,6 +1730,7 @@ json_t * user_auth_scheme_module_identify(struct config_module * config, const s
             j_return = json_pack("{si}", "result", G_ERROR);
           }
         } else {
+          y_log_message(Y_LOG_LEVEL_DEBUG, "user_auth_scheme_module_identify certificate - use-scheme-storage isn't set");
           j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
         }
       }
