@@ -31,7 +31,7 @@
 #include <orcania.h>
 #include <rhonabwy.h>
 #include <iddawc.h>
-#include "../glewlwyd-common.h"
+#include "glewlwyd-common.h"
 
 #define GLEWLWYD_SCHEME_OAUTH2_REGISTRATION_TABLE "gs_oauth2_registration"
 #define GLEWLWYD_SCHEME_OAUTH2_SESSION_TABLE "gs_oauth2_session"
@@ -262,7 +262,7 @@ static json_t * complete_session_identify(struct config_module * config, struct
                       ret = G_ERROR_PARAM;
                     }
                   } else {
-                    if ((res = i_load_userinfo(&i_session)) == I_OK && i_session.j_userinfo != NULL) {
+                    if ((res = i_get_userinfo(&i_session, 0)) == I_OK && i_session.j_userinfo != NULL) {
                       if (json_string_length((json_object_get(i_session.j_userinfo, json_string_value(json_object_get(j_provider, "userid_property")))))) {
                         sub = o_strdup(json_string_value(json_object_get(i_session.j_userinfo, json_string_value(json_object_get(j_provider, "userid_property")))));
                         ret = o_strlen(sub)?G_OK:G_ERROR_PARAM;
@@ -276,7 +276,7 @@ static json_t * complete_session_identify(struct config_module * config, struct
                     } else if (res == I_ERROR_PARAM || res == I_ERROR_SERVER || res == I_ERROR_UNAUTHORIZED || i_session.j_userinfo == NULL) {
                       ret = G_ERROR_PARAM;
                     } else {
-                      y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_identify - Error i_load_userinfo (1)");
+                      y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_identify - Error i_get_userinfo (1)");
                       ret = G_ERROR;
                     }
                   }
@@ -288,7 +288,7 @@ static json_t * complete_session_identify(struct config_module * config, struct
                 }
                 break;
               case I_RESPONSE_TYPE_TOKEN:
-                if ((res = i_load_userinfo(&i_session)) == I_OK && i_session.j_userinfo != NULL) {
+                if ((res = i_get_userinfo(&i_session, 0)) == I_OK && i_session.j_userinfo != NULL) {
                   if (json_string_length(json_object_get(i_session.j_userinfo, json_string_value(json_object_get(j_provider, "userid_property"))))) {
                     sub = o_strdup(json_string_value(json_object_get(i_session.j_userinfo, json_string_value(json_object_get(j_provider, "userid_property")))));
                     ret = o_strlen(sub)?G_OK:G_ERROR_PARAM;
@@ -302,7 +302,7 @@ static json_t * complete_session_identify(struct config_module * config, struct
                 } else if (res == I_ERROR_PARAM || res == I_ERROR_SERVER || res == I_ERROR_UNAUTHORIZED || i_session.j_userinfo == NULL) {
                   ret = G_ERROR_PARAM;
                 } else {
-                  y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_identify - Error i_load_userinfo (2)");
+                  y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_identify - Error i_get_userinfo (2)");
                   ret = G_ERROR;
                 }
                 break;
@@ -356,6 +356,7 @@ static json_t * complete_session_identify(struct config_module * config, struct
                   json_decref(j_query);
                   if (res != H_OK) {
                     y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_identify - Error executing j_query (1)");
+                    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                     j_return = json_pack("{si}", "result", G_ERROR_DB);
                   } else {
                     j_return = json_pack("{sisO}", "result", G_OK, "username", json_object_get(json_array_get(j_result_sub, 0), "username"));
@@ -375,6 +376,7 @@ static json_t * complete_session_identify(struct config_module * config, struct
                   json_decref(j_query);
                   if (res != H_OK) {
                     y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_identify - Error executing j_query (2)");
+                    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                     j_return = json_pack("{si}", "result", G_ERROR_DB);
                   } else {
                     j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
@@ -383,6 +385,7 @@ static json_t * complete_session_identify(struct config_module * config, struct
                 json_decref(j_result_sub);
               } else {
                 y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_identify - Error executing j_query (3)");
+                config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                 j_return = json_pack("{si}", "result", G_ERROR_DB);
               }
             } else {
@@ -399,6 +402,7 @@ static json_t * complete_session_identify(struct config_module * config, struct
               json_decref(j_query);
               if (res != H_OK) {
                 y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_identify - Error executing j_query (4)");
+                config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                 j_return = json_pack("{si}", "result", G_ERROR_DB);
               } else {
                 j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
@@ -428,6 +432,7 @@ static json_t * complete_session_identify(struct config_module * config, struct
     json_decref(j_result);
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_identify - Error executing j_query (5)");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     j_return = json_pack("{si}", "result", G_ERROR_DB);
   }
   return j_return;
@@ -482,6 +487,7 @@ static json_t * add_session_identify(struct config_module * config, struct _oaut
                 j_return = json_pack("{siss}", "result", G_OK, "session", i_get_str_parameter(&i_session, I_OPT_REDIRECT_TO));
               } else {
                 y_log_message(Y_LOG_LEVEL_ERROR, "add_session_for_user - Error executing j_query (2)");
+                config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                 j_return = json_pack("{si}", "result", G_ERROR_DB);
               }
             } else {
@@ -593,6 +599,7 @@ static json_t * add_session_for_user(struct config_module * config, struct _oaut
                   j_return = json_pack("{siss}", "result", G_OK, "session", i_get_str_parameter(&i_session, I_OPT_REDIRECT_TO));
                 } else {
                   y_log_message(Y_LOG_LEVEL_ERROR, "add_session_for_user - Error executing j_query (2)");
+                  config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                   j_return = json_pack("{si}", "result", G_ERROR_DB);
                 }
               } else {
@@ -625,6 +632,7 @@ static json_t * add_session_for_user(struct config_module * config, struct _oaut
     }
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "add_session_for_user - Error executing j_query");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     j_return = json_pack("{si}", "result", G_ERROR_DB);
   }
   return j_return;
@@ -659,6 +667,7 @@ static json_t * get_last_session_for_registration(struct config_module * config,
     json_decref(j_result);
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "get_last_session_for_registration - Error executing j_query");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     j_return = json_pack("{si}", "result", G_ERROR_DB);
   }
   return j_return;
@@ -712,6 +721,7 @@ static json_t * get_registration_for_user(struct config_module * config, struct
     json_decref(j_result);
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "get_registration_for_user - Error executing j_query");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     j_return = json_pack("{si}", "result", G_ERROR_DB);
   }
   return j_return;
@@ -784,10 +794,12 @@ static json_t * add_registration_for_user(struct config_module * config, struct
                     j_return = json_pack("{siss}", "result", G_OK, "registration", i_get_str_parameter(&i_session, I_OPT_REDIRECT_TO));
                   } else {
                     y_log_message(Y_LOG_LEVEL_ERROR, "add_registration_for_user - Error executing j_query (2)");
+                    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                     j_return = json_pack("{si}", "result", G_ERROR_DB);
                   }
                 } else {
                   y_log_message(Y_LOG_LEVEL_ERROR, "add_registration_for_user - Error executing j_query (1)");
+                  config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                   j_return = json_pack("{si}", "result", G_ERROR_DB);
                 }
               } else {
@@ -847,6 +859,7 @@ static int delete_registration_for_user(struct config_module * config, struct _o
     ret = G_OK;
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "delete_registration_for_user - Error executing j_query");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     ret = G_ERROR_DB;
   }
   return ret;
@@ -910,7 +923,7 @@ static int complete_session_for_user(struct config_module * config, const char *
                       ret = G_ERROR_PARAM;
                     }
                   } else {
-                    if ((res = i_load_userinfo(&i_session)) == I_OK && i_session.j_userinfo != NULL) {
+                    if ((res = i_get_userinfo(&i_session, 0)) == I_OK && i_session.j_userinfo != NULL) {
                       if (json_string_length((json_object_get(i_session.j_userinfo, json_string_value(json_object_get(j_provider, "userid_property")))))) {
                         sub = o_strdup(json_string_value(json_object_get(i_session.j_userinfo, json_string_value(json_object_get(j_provider, "userid_property")))));
                         ret = o_strlen(sub)?G_OK:G_ERROR_PARAM;
@@ -924,7 +937,7 @@ static int complete_session_for_user(struct config_module * config, const char *
                     } else if (res == I_ERROR_PARAM || res == I_ERROR_SERVER || res == I_ERROR_UNAUTHORIZED || i_session.j_userinfo == NULL) {
                       ret = G_ERROR_PARAM;
                     } else {
-                      y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_for_user - Error i_load_userinfo (1)");
+                      y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_for_user - Error i_get_userinfo (1)");
                       ret = G_ERROR;
                     }
                   }
@@ -936,7 +949,7 @@ static int complete_session_for_user(struct config_module * config, const char *
                 }
                 break;
               case I_RESPONSE_TYPE_TOKEN:
-                if ((res = i_load_userinfo(&i_session)) == I_OK && i_session.j_userinfo != NULL) {
+                if ((res = i_get_userinfo(&i_session, 0)) == I_OK && i_session.j_userinfo != NULL) {
                   if (json_string_length(json_object_get(i_session.j_userinfo, json_string_value(json_object_get(j_provider, "userid_property"))))) {
                     sub = o_strdup(json_string_value(json_object_get(i_session.j_userinfo, json_string_value(json_object_get(j_provider, "userid_property")))));
                     ret = o_strlen(sub)?G_OK:G_ERROR_PARAM;
@@ -950,7 +963,7 @@ static int complete_session_for_user(struct config_module * config, const char *
                 } else if (res == I_ERROR_PARAM || res == I_ERROR_SERVER || res == I_ERROR_UNAUTHORIZED || i_session.j_userinfo == NULL) {
                   ret = G_ERROR_PARAM;
                 } else {
-                  y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_for_user - Error i_load_userinfo (2)");
+                  y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_for_user - Error i_get_userinfo (2)");
                   ret = G_ERROR;
                 }
                 break;
@@ -986,6 +999,7 @@ static int complete_session_for_user(struct config_module * config, const char *
                 json_decref(j_query);
                 if (res != H_OK) {
                   y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_for_user - Error executing j_query (2)");
+                  config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                   ret = G_ERROR_DB;
                 }
               } else {
@@ -1006,6 +1020,7 @@ static int complete_session_for_user(struct config_module * config, const char *
               json_decref(j_query);
               if (res != H_OK) {
                 y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_for_user - Error executing j_query (3)");
+                config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                 ret = G_ERROR_DB;
               }
             } else {
@@ -1022,6 +1037,7 @@ static int complete_session_for_user(struct config_module * config, const char *
               json_decref(j_query);
               if (res != H_OK) {
                 y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_for_user - Error executing j_query (3)");
+                config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                 ret = G_ERROR_DB;
               }
             }
@@ -1048,6 +1064,7 @@ static int complete_session_for_user(struct config_module * config, const char *
     json_decref(j_result);
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "complete_session_for_user - Error executing j_query (1)");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     ret = G_ERROR_DB;
   }
   return ret;
@@ -1192,9 +1209,10 @@ json_t * user_auth_scheme_module_init(struct config_module * config, json_t * j_
                                                                       I_OPT_CLIENT_SECRET, json_string_value(json_object_get(j_element, "client_secret")),
                                                                       I_OPT_REDIRECT_URI, json_string_value(json_object_get(j_parameters, "redirect_uri")),
                                                                       I_OPT_SCOPE, is_oidc?"openid":json_string_value(json_object_get(j_element, "scope")),
+                                                                      I_OPT_TOKEN_METHOD, I_TOKEN_AUTH_METHOD_SECRET_BASIC,
                                                                       I_OPT_NONE) != I_OK) {
                   y_log_message(Y_LOG_LEVEL_ERROR, "user_auth_scheme_module_init oauth2 - Error setting parameters for provider %s", json_string_value(json_object_get(j_element, "name")));
-                } else if (i_load_openid_config(&i_session) != I_OK) {
+                } else if (i_get_openid_config(&i_session) != I_OK) {
                   y_log_message(Y_LOG_LEVEL_ERROR, "user_auth_scheme_module_init oauth2 - Error loading openid-configuration for provider %s", json_string_value(json_object_get(j_element, "name")));
                 } else if ((j_export = i_export_session_json_t(&i_session)) == NULL) {
                   y_log_message(Y_LOG_LEVEL_ERROR, "user_auth_scheme_module_init oauth2 - Error exporting session for provider %s", json_string_value(json_object_get(j_element, "name")));
@@ -1222,6 +1240,7 @@ json_t * user_auth_scheme_module_init(struct config_module * config, json_t * j_
                                                                       I_OPT_CLIENT_SECRET, json_string_value(json_object_get(j_element, "client_secret")),
                                                                       I_OPT_REDIRECT_URI, json_string_value(json_object_get(j_parameters, "redirect_uri")),
                                                                       I_OPT_SCOPE, is_oidc?"openid":json_string_value(json_object_get(j_element, "scope")),
+                                                                      I_OPT_TOKEN_METHOD, I_TOKEN_AUTH_METHOD_SECRET_BASIC,
                                                                       I_OPT_NONE) != I_OK) {
                   y_log_message(Y_LOG_LEVEL_ERROR, "user_auth_scheme_module_init oauth2 - Error setting parameters for provider %s", json_string_value(json_object_get(j_element, "name")));
                 } else if ((j_export = i_export_session_json_t(&i_session)) == NULL) {
