@@ -32,7 +32,7 @@
 #include <liboath/oath.h>
 #include <yder.h>
 #include <orcania.h>
-#include "../glewlwyd-common.h"
+#include "glewlwyd-common.h"
 
 #define GLEWLWYD_TABLE_OTP "gs_otp"
 
@@ -184,6 +184,7 @@ static json_t * get_otp(struct config_module * config, json_t * j_params, const
     json_decref(j_result);
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "get_otp - Error executing j_query");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     j_return = json_pack("{si}", "result", G_ERROR_DB);
   }
   return j_return;
@@ -228,6 +229,7 @@ static int update_otp(struct config_module * config, json_t * j_params, const ch
     ret = G_OK;
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "update_otp - Error executing j_query");
+    config->glewlwyd_module_callback_metrics_increment_counter(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     ret = G_ERROR_DB;
   }
   json_decref(j_query);
@@ -293,7 +295,7 @@ static int set_otp(struct config_module * config, json_t * j_params, const char
                             json_object_get(j_params, "mod_name"),
                             "gso_last_used",
                               "raw",
-                              SWITCH_DB_TYPE(config->conn->type, "FROM_UNIXTIME(0)", "0", "TO_TIMESTAMP(0)")); // Let's hope no one will use Glewlwyd before January 1st, 1970!
+                              SWITCH_DB_TYPE(config->conn->type, "FROM_UNIXTIME(1)", "1", "TO_TIMESTAMP(1)")); // Let's hope no one will use Glewlwyd before January 1st, 1970!
       res = h_insert(config->conn, j_query, NULL);
       json_decref(j_query);
       if (res == H_OK) {
