@@ -61,6 +61,7 @@ json_t * get_session_scheme(struct config_elements * config, json_int_t gus_id)
     j_return = json_pack("{siso}", "result", G_OK, "scheme", j_result);
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "get_session_scheme - Error executing j_query");
+    glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     j_return = json_pack("{si}", "result", G_ERROR_DB);
   }
   return j_return;
@@ -128,6 +129,7 @@ json_t * get_session_for_username(struct config_elements * config, const char *
       json_decref(j_result);
     } else {
       y_log_message(Y_LOG_LEVEL_ERROR, "get_session_for_username - Error executing j_query");
+      glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
       j_return = json_pack("{si}", "result", G_ERROR_DB);
     }
     o_free(session_uid_hash);
@@ -206,6 +208,7 @@ json_t * get_users_for_session(struct config_elements * config, const char * ses
         json_decref(j_result);
       } else {
         y_log_message(Y_LOG_LEVEL_ERROR, "get_users_for_session - Error executing j_query");
+        glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
         j_return = json_pack("{si}", "result", G_ERROR_DB);
       }
     } else {
@@ -266,6 +269,7 @@ json_t * get_current_user_for_session(struct config_elements * config, const cha
         json_decref(j_result);
       } else {
         y_log_message(Y_LOG_LEVEL_ERROR, "get_current_user_for_session - Error executing j_query");
+        glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
         j_return = json_pack("{si}", "result", G_ERROR_DB);
       }
     } else if (session_uid == NULL) {
@@ -347,10 +351,12 @@ int user_session_update(struct config_elements * config, const char * session_ui
           j_session = get_session_for_username(config, session_uid, username);
         } else {
           y_log_message(Y_LOG_LEVEL_ERROR, "user_session_update - Error h_insert session");
+          glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
           j_session = json_pack("{si}", "result", G_ERROR_DB);
         }
       } else {
         y_log_message(Y_LOG_LEVEL_ERROR, "user_session_update - Error h_update session (0)");
+        glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
         j_session = json_pack("{si}", "result", G_ERROR_DB);
       }
     } else {
@@ -407,10 +413,12 @@ int user_session_update(struct config_elements * config, const char * session_ui
           j_session = get_session_for_username(config, session_uid, username);
         } else {
           y_log_message(Y_LOG_LEVEL_ERROR, "user_session_update - Error h_update session (2)");
+          glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
           j_session = json_pack("{si}", "result", G_ERROR_DB);
         }
       } else {
         y_log_message(Y_LOG_LEVEL_ERROR, "user_session_update - Error h_update session (1)");
+        glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
         j_session = json_pack("{si}", "result", G_ERROR_DB);
       }
     }
@@ -471,10 +479,12 @@ int user_session_update(struct config_elements * config, const char * session_ui
                 ret = G_OK;
               } else {
                 y_log_message(Y_LOG_LEVEL_ERROR, "user_session_update - Error executing j_query (1)");
+                glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
                 ret = G_ERROR_DB;
               }
             } else {
               y_log_message(Y_LOG_LEVEL_ERROR, "user_session_update - Error executing j_query (2)");
+              glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
               ret = G_ERROR_DB;
             }
           } else {
@@ -531,10 +541,12 @@ int user_session_update(struct config_elements * config, const char * session_ui
               ret = G_OK;
             } else {
               y_log_message(Y_LOG_LEVEL_ERROR, "user_session_update - Error executing j_query (3)");
+              glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
               ret = G_ERROR_DB;
             }
           } else {
             y_log_message(Y_LOG_LEVEL_ERROR, "user_session_update - Error executing j_query (4)");
+            glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
             ret = G_ERROR_DB;
           }
         }
@@ -596,6 +608,7 @@ int user_session_delete(struct config_elements * config, const char * session_ui
           ret = G_OK;
         } else {
           y_log_message(Y_LOG_LEVEL_ERROR, "user_session_delete - Error executing j_query (2)");
+          glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
           ret = G_ERROR_DB;
         }
       } else {
@@ -603,6 +616,7 @@ int user_session_delete(struct config_elements * config, const char * session_ui
       }
     } else {
       y_log_message(Y_LOG_LEVEL_ERROR, "user_session_delete - Error executing j_query (1)");
+      glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
       ret = G_ERROR_DB;
     }
     o_free(session_uid_hash);
@@ -681,49 +695,54 @@ json_t * get_user_session_list(struct config_elements * config, const char * use
     json_decref(j_result);
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "user_session_delete - Error executing j_query");
+    glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
     j_return = json_pack("{si}", "result", G_ERROR_DB);
   }
   return j_return;
 }
 
 int delete_user_session_from_hash(struct config_elements * config, const char * username, const char * session_hash) {
-  json_t * j_query, * j_result;
-  int res, ret;
+  json_t * j_query, * j_result, * j_element = NULL;
+  int res, ret = G_OK;
   unsigned char session_hash_dec[128];
-  size_t session_hash_dec_len = 0;
+  size_t session_hash_dec_len = 0, index = 0;
   
-  if (o_base64url_2_base64((unsigned char *)session_hash, o_strlen(session_hash), session_hash_dec, &session_hash_dec_len)) {
-    j_query = json_pack("{sss[s]s{ss%ss}}",
-                        "table",
-                        GLEWLWYD_TABLE_USER_SESSION,
-                        "columns",
-                          "gus_id",
-                        "where",
-                          "gus_session_hash",
-                          session_hash_dec,
-                          session_hash_dec_len,
-                          "gus_username",
-                          username);
+  j_query = json_pack("{sss[s]s{ss}}",
+                      "table",
+                      GLEWLWYD_TABLE_USER_SESSION,
+                      "columns",
+                        "gus_id",
+                      "where",
+                        "gus_username", username);
+  if (session_hash != NULL) {
+    if (o_base64url_2_base64((unsigned char *)session_hash, o_strlen(session_hash), session_hash_dec, &session_hash_dec_len)) {
+      json_object_set_new(json_object_get(j_query, "where"), "gus_session_hash", json_stringn((const char *)session_hash_dec, session_hash_dec_len));
+    } else {
+      y_log_message(Y_LOG_LEVEL_ERROR, "delete_user_session_from_hash - Error o_base64url_2_base64");
+      ret = G_ERROR_PARAM;
+    }
+  }
+  if (ret == G_OK) {
     res = h_select(config->conn, j_query, &j_result, NULL);
     json_decref(j_query);
     if (res == H_OK) {
       if (json_array_size(j_result)) {
-        j_query = json_pack("{sss{si}s{sO}}",
-                            "table",
-                            GLEWLWYD_TABLE_USER_SESSION,
-                            "set",
-                              "gus_enabled",
-                              0,
-                            "where",
-                              "gus_id",
-                              json_object_get(json_array_get(j_result, 0), "gus_id"));
-        res = h_update(config->conn, j_query, NULL);
-        json_decref(j_query);
-        if (res == H_OK) {
-          ret = G_OK;
-        } else {
-          y_log_message(Y_LOG_LEVEL_ERROR, "delete_user_session_from_hash - Error executing j_query (2)");
-          ret = G_ERROR_DB;
+        json_array_foreach(j_result, index, j_element) {
+          j_query = json_pack("{sss{si}s{sO}}",
+                              "table",
+                              GLEWLWYD_TABLE_USER_SESSION,
+                              "set",
+                                "gus_enabled",
+                                0,
+                              "where",
+                                "gus_id", json_object_get(j_element, "gus_id"));
+          res = h_update(config->conn, j_query, NULL);
+          json_decref(j_query);
+          if (res != H_OK) {
+            y_log_message(Y_LOG_LEVEL_ERROR, "delete_user_session_from_hash - Error executing j_query (2)");
+            glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
+            ret = G_ERROR_DB;
+          }
         }
       } else {
         ret = G_ERROR_NOT_FOUND;
@@ -731,11 +750,9 @@ int delete_user_session_from_hash(struct config_elements * config, const char *
       json_decref(j_result);
     } else {
       y_log_message(Y_LOG_LEVEL_ERROR, "delete_user_session_from_hash - Error executing j_query (1)");
+      glewlwyd_metrics_increment_counter_va(config, GLWD_METRICS_DATABSE_ERROR, 1, NULL);
       ret = G_ERROR_DB;
     }
-  } else {
-    y_log_message(Y_LOG_LEVEL_ERROR, "delete_user_session_from_hash - Error o_base64url_2_base64");
-    ret = G_ERROR_PARAM;
   }
   return ret;
 }
