@@ -3,12 +3,31 @@ DELETE FROM g_user_session;
 DELETE FROM g_scope_group_auth_scheme_module_instance;
 DELETE FROM g_scope_group;
 DELETE FROM g_scope;
+DELETE FROM g_user_middleware_module_instance;
 DELETE FROM g_user_module_instance;
 DELETE FROM g_user_auth_scheme_module_instance;
 DELETE FROM g_client_module_instance;
 DELETE FROM g_client_module_instance;
 DELETE FROM g_plugin_module_instance;
 DELETE FROM gpr_session;
+DELETE FROM gs_webauthn_assertion;
+DELETE FROM gs_webauthn_credential;
+DELETE FROM gs_webauthn_user;
+DELETE FROM gpo_ciba;
+DELETE FROM gpo_par;
+DELETE FROM gpo_dpop;
+DELETE FROM gpo_device_authorization;
+DELETE FROM gpo_client_token_request;
+DELETE FROM gpo_client_registration;
+DELETE FROM gpo_subject_identifier;
+DELETE FROM gpo_id_token;
+DELETE FROM gpo_access_token;
+DELETE FROM gpo_refresh_token;
+DELETE FROM gpo_code;
+DELETE FROM gpg_device_authorization;
+DELETE FROM gpg_access_token;
+DELETE FROM gpg_refresh_token;
+DELETE FROM gpg_code;
 
 INSERT INTO g_user_module_instance (gumi_module, gumi_name, gumi_display_name, gumi_order, gumi_parameters) VALUES ('mock', 'mock', 'Mock user module', 0, '{"username-prefix":"","password":"password"}');
 INSERT INTO g_user_auth_scheme_module_instance (guasmi_module, guasmi_name, guasmi_display_name, guasmi_expiration, guasmi_parameters) VALUES ('mock', 'mock_scheme_42', 'Mock 42', 600, '{"mock-value":"42"}');
