@@ -15,14 +15,14 @@
 
 #define SERVER_URI "http://localhost:4593/api"
 #define USERNAME "user1"
-#define PASSWORD "MyUser1Password!"
-#define SCOPE_LIST "scope1 scope2"
+#define PASSWORD "password"
+#define SCOPE_LIST "g_profile scope3"
 
 char * refresh_token;
 
-START_TEST(test_glwd_refresh_token_token_invalid)
+START_TEST(test_oauth2_refresh_token_token_invalid)
 {
-  char * url = msprintf("%s/token/", SERVER_URI);
+  char * url = msprintf("%s/glwd/token/", SERVER_URI);
   struct _u_map body;
   u_map_init(&body);
   u_map_put(&body, "grant_type", "refresh_token");
@@ -35,9 +35,9 @@ START_TEST(test_glwd_refresh_token_token_invalid)
 }
 END_TEST
 
-START_TEST(test_glwd_refresh_token_ok)
+START_TEST(test_oauth2_refresh_token_ok)
 {
-  char * url = msprintf("%s/token/", SERVER_URI);
+  char * url = msprintf("%s/glwd/token/", SERVER_URI);
   struct _u_map body;
   u_map_init(&body);
   u_map_put(&body, "grant_type", "refresh_token");
@@ -55,10 +55,10 @@ static Suite *glewlwyd_suite(void)
   Suite *s;
   TCase *tc_core;
 
-  s = suite_create("Glewlwyd refresh token");
-  tc_core = tcase_create("test_glwd_refresh_token");
-  tcase_add_test(tc_core, test_glwd_refresh_token_token_invalid);
-  tcase_add_test(tc_core, test_glwd_refresh_token_ok);
+  s = suite_create("Glewlwyd oauth2 refresh token");
+  tc_core = tcase_create("test_oauth2_refresh_token");
+  tcase_add_test(tc_core, test_oauth2_refresh_token_token_invalid);
+  tcase_add_test(tc_core, test_oauth2_refresh_token_ok);
   tcase_set_timeout(tc_core, 30);
   suite_add_tcase(s, tc_core);
 
@@ -80,13 +80,13 @@ int main(int argc, char *argv[])
   ulfius_init_request(&auth_req);
   ulfius_init_response(&auth_resp);
   auth_req.http_verb = strdup("POST");
-  auth_req.http_url = msprintf("%s/token/", SERVER_URI);
+  auth_req.http_url = msprintf("%s/glwd/token/", SERVER_URI);
   u_map_put(auth_req.map_post_body, "grant_type", "password");
   u_map_put(auth_req.map_post_body, "username", USERNAME);
   u_map_put(auth_req.map_post_body, "password", PASSWORD);
   u_map_put(auth_req.map_post_body, "scope", SCOPE_LIST);
   res = ulfius_send_http_request(&auth_req, &auth_resp);
-  if (res == U_OK) {
+  if (res == U_OK && auth_resp.status == 200) {
     json_t * json_body = ulfius_get_json_body_response(&auth_resp, NULL);
     refresh_token = o_strdup(json_string_value(json_object_get(json_body, "refresh_token")));
     y_log_message(Y_LOG_LEVEL_INFO, "User %s authenticated", USERNAME);
@@ -102,7 +102,7 @@ int main(int argc, char *argv[])
   number_failed = srunner_ntests_failed(sr);
   srunner_free(sr);
   
-  free(refresh_token);
+  o_free(refresh_token);
   
   return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
