@@ -331,7 +331,11 @@ class GlwdOauth2Params extends Component {
   
   render() {
     var keyJsx, certJsx, scopeOverrideList = [], scopeList = [], additionalParametersList = [];
-    var baseApiUrl = document.location.href.split('?')[0].split('#')[0] + this.state.config.api_prefix + "/" + (this.state.mod.name||"");
+    var baseApiUrl = document.location.href.split('?')[0].split('#')[0];
+    if (baseApiUrl.endsWith(this.state.config.AdminUrl)) {
+      baseApiUrl = baseApiUrl.substring(0, baseApiUrl.length-this.state.config.AdminUrl.length-1) + "/";
+    }
+    baseApiUrl += this.state.config.api_prefix + "/" + (this.state.mod.name||"");
     var urlAuth = baseApiUrl + "/auth", urlToken = baseApiUrl + "/token", urlProfile = baseApiUrl + "/profile";
     if (this.state.mod.parameters["jwt-type"] === "sha") {
       keyJsx =
@@ -475,7 +479,15 @@ class GlwdOauth2Params extends Component {
     });
     this.state.mod.parameters["introspection-revocation-auth-scope"].forEach((scope, index) => {
       if (this.state.mod.parameters["introspection-revocation-allowed"]) {
-        defaultScopeList.push(<a className="btn-icon-right" href="#" onClick={(e) => this.deleteScope(e, index)} key={index} ><span className="badge badge-primary">{scope}<span className="badge badge-light btn-icon-right"><i className="fas fa-times"></i></span></span></a>);
+        defaultScopeList.push(
+          <a className="btn-icon-right" href="#" onClick={(e) => this.deleteScope(e, index)} key={index} >
+            <span className="badge badge-primary">{scope}
+              <span className="badge badge-light btn-icon-right">
+                <i className="fas fa-times"></i>
+              </span>
+            </span>
+          </a>
+        );
       } else {
         defaultScopeList.push(<span key={index} className="badge badge-primary btn-icon-right">{scope}</span>);
       }
