@@ -10,9 +10,11 @@ import PasswordForm from './PasswordForm';
 import NoPasswordForm from './NoPasswordForm';
 import SelectAccount from './SelectAccount';
 import EndSession from './EndSession';
+import SingleLogout from './SingleLogout';
 import SessionClosed from './SessionClosed';
 import DeviceAuth from './DeviceAuth';
 import ResetCredentials from './ResetCredentials';
+import CibaMessage from './CibaMessage';
 
 import Message from '../Modal/Message';
 
@@ -42,9 +44,11 @@ class App extends Component {
       forceShowGrant: false,
       selectAccount: false,
       endSession: false,
+      singleLogout: false,
       sessionClosed: false,
       deviceAuth: false,
       login_hint: props.config.params.login_hint||"",
+      ciba_message: props.config.params.ciba_message,
       errorScopesUnavailable: false,
       infoSomeScopeUnavailable: false,
       errorScheme: false,
@@ -52,6 +56,7 @@ class App extends Component {
       resetCredentials: [],
       resetCredentialsShow: false,
       authDetails: [],
+      pluginList: [],
       messageModal: {
         title: "",
         label: "",
@@ -95,7 +100,7 @@ class App extends Component {
       } else if (message.type === "newUserScheme") {
         this.setState({scheme: message.scheme, identify: message.identify});
       } else if (message.type === "SessionClosed") {
-        this.setState({endSession: false, sessionClosed: true});
+        this.setState({endSession: false, singleLogout: false, sessionClosed: true});
       } else if (message.type === "SessionReopen") {
         this.setState({sessionClosed: false, newUser: true, currentUser: false, login_hint: ""}, () => {
           this.initProfile(true);
@@ -175,8 +180,14 @@ class App extends Component {
           newState.forceShowGrant = true;
         } else if (this.state.prompt === "select_account") {
           newState.selectAccount = true;
+        } else if (this.state.prompt === "single_logout") {
+          newState.endSession = false;
+          newState.singleLogout = true;
+          newState.newUser = false;
+          newState.currentUser = false;
         } else if (this.state.prompt === "end_session") {
           newState.endSession = true;
+          newState.singleLogout = false;
           newState.newUser = false;
           newState.currentUser = false;
         } else if (this.state.prompt && this.state.prompt.substring(0, 6) === "device") {
@@ -196,6 +207,15 @@ class App extends Component {
         } else {
           this.setState({showGrantAsterisk: false});
         }
+        apiManager.glewlwydRequest("/profile/plugin")
+        .then((res) => {
+          this.setState({pluginList: res});
+        })
+        .fail((err) => {
+          if (err.status != 401) {
+            messageDispatcher.sendMessage('Notification', {type: "warning", message: i18next.t("error-api-connect")});
+          }
+        });
       });
     })
     .fail((error) => {
@@ -375,7 +395,7 @@ class App extends Component {
                    passwordRequired: passwordRequired,
                    schemeListRequired: schemeListRequired,
                    scheme: scheme,
-                   errorScheme: (!scheme && !canContinue),
+                   errorScheme: (!scheme && !canContinue && !passwordRequired),
                    mustRegisterScheme: mustRegisterScheme});
   }
 
@@ -400,16 +420,20 @@ class App extends Component {
 
 	render() {
     if (this.state.config) {
-      var body = "", message, scopeUnavailable;
+      var body = "", message, ciba_binding_message, ciba_login_hint, scopeUnavailable;
       if (this.state.loaded) {
         if (this.state.resetCredentialsShow) {
           body = <ResetCredentials config={this.state.config} resetCredentials={this.state.resetCredentials}/>;
         } else if (this.state.endSession) {
-          body = <EndSession config={this.state.config} userList={this.state.userList} currentUser={this.state.currentUser}/>;
+          body = <EndSession config={this.state.config} userList={this.state.userList} currentUser={this.state.currentUser} pluginList={this.state.pluginList}/>;
+        } else if (this.state.singleLogout) {
+          body = <SingleLogout config={this.state.config} userList={this.state.userList} currentUser={this.state.currentUser} pluginList={this.state.pluginList}/>;
         } else if (this.state.sessionClosed) {
           body = <SessionClosed config={this.state.config}/>;
         } else if (this.state.deviceAuth) {
           body = <DeviceAuth config={this.state.config} userList={this.state.userList} currentUser={this.state.currentUser}/>;
+        } else if (this.state.ciba_message) {
+          body = <CibaMessage config={this.state.config} ciba_message={this.state.ciba_message}/>;
         } else {
           if (this.state.mustRegisterScheme && !this.state.errorScopesUnavailable) {
             message = <div className="alert alert-warning" role="alert">{i18next.t("login.warning-not-registered-scheme")}</div>
@@ -425,6 +449,12 @@ class App extends Component {
             }
             message = <div>{noCallback}{noScope}</div>;
           }
+          if (this.state.config.params.ciba_binding_message) {
+            ciba_binding_message = <div className="alert alert-info" role="alert">{this.state.config.params.ciba_binding_message}</div>;
+          }
+          if (this.state.config.params.ciba_login_hint && this.state.config.params.login_hint !== this.state.currentUser.username) {
+            ciba_login_hint = <div className="alert alert-success" role="alert">{i18next.t("login.ciba-login-hint", {username: this.state.config.params.login_hint})}</div>;
+          }
           if ((this.state.newUser || this.state.passwordRequired)) {
             if (!this.state.scheme) {
               body = <PasswordForm config={this.state.config}
@@ -491,6 +521,8 @@ class App extends Component {
                 </div>
               </nav>
             </div>
+            {ciba_binding_message}
+            {ciba_login_hint}
             {message}
             <div className="card-body">
               {scopeUnavailable}
@@ -511,6 +543,7 @@ class App extends Component {
                        registration={this.state.registration}
                        resetCredentials={this.state.resetCredentials}
                        resetCredentialsShow={this.state.resetCredentialsShow} 
+                       pluginList={this.state.pluginList}
                        sessionClosed={this.state.sessionClosed}/>
             </div>
           </div>
