@@ -61,6 +61,7 @@ class App extends Component {
         }
       },
       sessionList: [],
+      clientGrantList: [],
       invalidCredentialMessage: false,
       invalidDelegateMessage: false,
       tokenParsed: false,
@@ -69,13 +70,15 @@ class App extends Component {
       updateEmailModule: false,
       register: false,
       schemePrefix: ((props.config && props.config.params.register)?"/" + props.config.params.register + "/profile":"/profile"),
-      resetCredentials: 0
+      resetCredentials: 0,
+      cibaList: []
     };
-    
+
     this.fetchProfile = this.fetchProfile.bind(this);
     this.updateEmailCallback = this.updateEmailCallback.bind(this);
     this.refreshSession = this.refreshSession.bind(this);
-    
+    this.refreshClientGrant = this.refreshClientGrant.bind(this);
+
     messageDispatcher.subscribe('App', (message) => {
       if (message.type === 'nav') {
         if (message.page === "password") {
@@ -158,9 +161,11 @@ class App extends Component {
         }
       } else if (message.type === 'refreshSession') {
         this.refreshSession();
+      } else if (message.type === 'refreshClientGrant') {
+        this.refreshClientGrant();
       }
     });
-    
+
     if (this.state.config) {
       this.getRegisterConfig();
       if (this.state.config.params.updateEmail) {
@@ -193,7 +198,7 @@ class App extends Component {
       }
     }
   }
-  
+
   getRegisterConfig() {
     if (this.state.config.register) {
       this.state.config.register.forEach((register, index) => {
@@ -225,18 +230,22 @@ class App extends Component {
       });
     }
   }
-  
+
   getResetCredentialsConfig() {
     apiManager.glewlwydRequest("/" + this.state.config.params.resetCredentials + "/reset-credentials/profile/")
     .then((profile) => {
       this.setState({profileList: [profile.user], loggedIn: true, schemeList: profile.scheme, schemePrefix: "/" + this.state.config.params.resetCredentials + "/reset-credentials/profile", resetCredentials: 1});
     })
     .fail((err) => {
-      messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
-      this.setState({resetCredentials: -1});
+      if (err.status === 401) {
+        this.setState({invalidCredentialMessage: true, resetCredentials: -1});
+        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("profile.requires-profile-scope")});
+      } else {
+        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+      }
     });
   }
-  
+
   fetchProfile() {
     if (!this.state.config.params.delegate) {
       apiManager.glewlwydRequest("/profile_list")
@@ -259,6 +268,7 @@ class App extends Component {
                 }
               });
               this.refreshSession();
+              this.refreshClientGrant();
             })
             .fail((error) => {
               this.setState({loggedIn: false, schemeList: [], invalidDelegateMessage: false, invalidCredentialMessage: true, profileUpdate: false}, () => {
@@ -305,47 +315,82 @@ class App extends Component {
       });
     }
   }
-  
+
   refreshSession() {
     apiManager.glewlwydRequest("/profile/session")
     .then((res) => {
-      this.setState({sessionList: res});
-    })
-    .fail(() => {
-      messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
-    });
-    apiManager.glewlwydRequest("/profile/plugin")
-    .then((res) => {
-      res.forEach((plugin) => {
-        if (plugin.module === "oauth2-glewlwyd") {
-          apiManager.glewlwydRequestSub("/" + plugin.name + "/profile/token" + (this.state.config.params.delegate?"?impersonate="+this.state.config.params.delegate:""))
-          .then((resPlugin) => {
-            var plugins = this.state.plugins;
-            plugins.oauth2[plugin.name] = resPlugin;
-            this.setState({plugins: plugins});
-          })
-          .fail((err) => {
-            messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+      this.setState({sessionList: res}, () => {
+        apiManager.glewlwydRequest("/profile/plugin")
+        .then((res) => {
+          res.forEach((plugin) => {
+            if (plugin.module === "oauth2-glewlwyd") {
+              apiManager.glewlwydRequestSub("/" + plugin.name + "/profile/token" + (this.state.config.params.delegate?"?impersonate="+this.state.config.params.delegate:""))
+              .then((resPlugin) => {
+                var plugins = this.state.plugins;
+                plugins.oauth2[plugin.name] = resPlugin;
+                this.setState({plugins: plugins});
+              })
+              .fail((err) => {
+                messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+              });
+            } else if (plugin.module === "oidc") {
+              apiManager.glewlwydRequestSub("/" + plugin.name + "/token" + (this.state.config.params.delegate?"?impersonate="+this.state.config.params.delegate:""))
+              .then((resPlugin) => {
+                var plugins = this.state.plugins;
+                plugins.oidc[plugin.name] = resPlugin;
+                this.setState({plugins: plugins});
+              })
+              .fail((err) => {
+                messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+              });
+              apiManager.glewlwydRequest("/" + plugin.name + "/ciba_user_list")
+              .then((cibaList) => {
+                this.setState({cibaList: cibaList});
+              })
+              .fail((err) => {
+                if (err.status !== 404) {
+                  messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+                }
+              });
+            }
           });
-        } else if (plugin.module === "oidc") {
-          apiManager.glewlwydRequestSub("/" + plugin.name + "/token" + (this.state.config.params.delegate?"?impersonate="+this.state.config.params.delegate:""))
-          .then((resPlugin) => {
-            var plugins = this.state.plugins;
-            plugins.oidc[plugin.name] = resPlugin;
-            this.setState({plugins: plugins});
-          })
-          .fail((err) => {
+          messageDispatcher.sendMessage('App', {type: "sessionComplete"});
+        })
+        .fail((err) => {
+          if (err.status === 401) {
+            this.setState({invalidCredentialMessage: true});
+            messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("profile.requires-profile-scope")});
+          } else {
             messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
-          });
-        }
+          }
+        });
       });
-      messageDispatcher.sendMessage('App', {type: "sessionComplete"});
     })
-    .fail(() => {
-      messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+    .fail((err) => {
+      if (err.status === 401) {
+        this.setState({invalidCredentialMessage: true, loggedIn: false, sessionList: [], clientGrantList: [], cibaList: []});
+        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("profile.requires-profile-scope")});
+      } else {
+        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+      }
     });
   }
   
+  refreshClientGrant() {
+    apiManager.glewlwydRequest("/profile/grant")
+    .then((res) => {
+      this.setState({clientGrantList: res});
+    })
+    .fail((err) => {
+      if (err.status === 401) {
+        this.setState({invalidCredentialMessage: true});
+        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("profile.requires-profile-scope")});
+      } else {
+        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+      }
+    });
+  }
+
   fetchRegistration() {
     this.state.registerPlugin.forEach((registerPlugin) => {
       if (registerPlugin.registration) {
@@ -416,14 +461,14 @@ class App extends Component {
                 });
               })
               .fail((err) => {
-                if (err.status != 401) {
+                if (err.status !== 401) {
                   messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
                 }
-                this.setState({registerProfile: false, registering: false});
+                this.setState({registerProfile: false, registering: false, invalidCredentialMessage: true});
               });
             })
             .fail((err) => {
-              if (err.status != 401) {
+              if (err.status !== 401) {
                 messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
               } else {
                 messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("profile.register-token-invalid")});
@@ -443,17 +488,22 @@ class App extends Component {
 
   updateEmailVerifyToken(module, token) {
     return apiManager.glewlwydRequest("/" + module + "/update-email/" + encodeURIComponent(token), "PUT")
-    .fail(() => {
-      messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+    .fail((err) => {
+      if (err.status === 401) {
+        this.setState({invalidCredentialMessage: true});
+        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("profile.requires-profile-scope")});
+      } else {
+        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+      }
     });
   }
-  
+
   showPasswordChangeNotification(result, data) {
     if (result) {
       messageDispatcher.sendMessage('Notification', {type: "success", message: i18next.t("profile.password-change-success")});
     }
   }
-  
+
   updateEmailCallback(result, value) {
     $("#editModal").modal("hide");
     if (result) {
@@ -467,15 +517,20 @@ class App extends Component {
           $("#messageModal").modal({keyboard: false, show: true});
         });
       })
-      .fail(() => {
-        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+      .fail((err) => {
+        if (err.status === 401) {
+          this.setState({invalidCredentialMessage: true});
+          messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("profile.requires-profile-scope")});
+        } else {
+          messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+        }
       });
     }
   }
-  
+
 	render() {
     if (this.state.config) {
-      var userJsx = "", invalidMessage;
+      var userJsx = "", invalidMessage, cibaListJsx = [];
       if (this.state.invalidCredentialMessage) {
         invalidMessage = <div className="alert alert-danger" role="alert">{i18next.t("profile.error-credential-message")}</div>
       } else if (this.state.invalidDelegateMessage) {
@@ -490,20 +545,39 @@ class App extends Component {
       } else {
         userJsx = <User config={this.state.config} profile={(this.state.profileList[0]||false)} pattern={this.state.config?this.state.config.pattern.user:false} profileUpdate={this.state.profileUpdate} loggedIn={this.state.loggedIn} updateEmail={this.state.updateEmail}/>
       }
+      this.state.cibaList.forEach((ciba, index) => {
+        var bindingMessage;
+        if (ciba.binding_message) {
+          bindingMessage = <div>{ciba.binding_message}</div>;
+        }
+        cibaListJsx.push(
+          <div className="alert alert-info" role="alert" key={index}>
+            {i18next.t("profile.ciba-connect-message", {client: ciba.client_name||ciba.client_id})}
+            {bindingMessage}
+            <div>
+              <a href={ciba.connect_uri}>{i18next.t("profile.ciba-connect-link")}</a>
+            </div>
+            <div>
+              <a href={ciba.cancel_uri}>{i18next.t("profile.ciba-cancel-link")}</a>
+            </div>
+          </div>
+        );
+      });
       return (
         <div aria-live="polite" aria-atomic="true" className="glwd-container">
           <div className="card center glwd-card" id="userCard" tabIndex="-1" role="dialog">
             <div className="card-header">
-              <Navbar active={this.state.curNav} 
-                      config={this.state.config} 
-                      loggedIn={this.state.loggedIn} 
-                      schemeList={this.state.schemeList} 
+              <Navbar active={this.state.curNav}
+                      config={this.state.config}
+                      loggedIn={this.state.loggedIn}
+                      schemeList={this.state.schemeList}
                       profileList={this.state.profileList}
                       dataHighlight={!this.state.registerProfile.password_set}
                       schemeHighlight={this.state.schemeHighlight}
                       registering={this.state.registering}/>
             </div>
             {invalidMessage}
+            {cibaListJsx}
             <div className="card-body">
               <div id="carouselBody" className="carousel slide" data-ride="carousel">
                 <div className="carousel-inner">
@@ -511,19 +585,19 @@ class App extends Component {
                     {userJsx}
                   </div>
                   <div className={"carousel-item" + (this.state.curNav==="session"?" active":"")}>
-                    <Session config={this.state.config} plugins={this.state.plugins} sessionList={this.state.sessionList}/>
+                    <Session config={this.state.config} plugins={this.state.plugins} sessionList={this.state.sessionList} clientGrantList={this.state.clientGrantList}/>
                   </div>
                   <div className={"carousel-item" + (this.state.curNav==="password"?" active":"")}>
-                    <Password config={this.state.config} 
-                              profile={(this.state.profileList[0]||false)} 
-                              loggedIn={this.state.loggedIn} 
-                              callback={this.showPasswordChangeNotification} 
+                    <Password config={this.state.config}
+                              profile={(this.state.profileList[0]||false)}
+                              loggedIn={this.state.loggedIn}
+                              callback={this.showPasswordChangeNotification}
                               registerPlugin={this.state.registerPlugin} />
                   </div>
                   <div className={"carousel-item" + (this.state.curNav!=="profile"&&this.state.curNav!=="session"&&this.state.curNav!=="password"?" active":"")}>
-                    <SchemePage config={this.state.config} 
-                                module={this.state.curNav} 
-                                name={this.state.module} 
+                    <SchemePage config={this.state.config}
+                                module={this.state.curNav}
+                                name={this.state.module}
                                 profile={(this.state.profileList[0]||false)}
                                 schemePrefix={this.state.schemePrefix}/>
                   </div>
@@ -534,10 +608,10 @@ class App extends Component {
           <Notification loggedIn={this.state.loggedIn||this.state.config.params.register}/>
           <Confirm title={this.state.confirmModal.title} message={this.state.confirmModal.message} callback={this.state.confirmModal.callback} />
           <Message title={this.state.messageModal.title} label={this.state.messageModal.label} message={this.state.messageModal.message} />
-          <Edit title={this.state.editModal.title} 
-                message={this.state.editModal.message} 
-                value={this.state.editModal.value} 
-                placeHolder={this.state.editModal.placeHolder} 
+          <Edit title={this.state.editModal.title}
+                message={this.state.editModal.message}
+                value={this.state.editModal.value}
+                placeholder={this.state.editModal.placeHolder}
                 callback={this.state.editModal.callback} />
         </div>
       );
