@@ -27,11 +27,23 @@ module.exports = {
 	},
 
 	devServer: {
-		contentBase: path.resolve(__dirname),
+		static: path.resolve(__dirname),
 		compress: true,
 		port: 3000,
 		host: 'localhost',
-		open: true
+		open: true,
+    proxy: {
+      '/api': {
+        target: 'http://localhost:4593/',
+        secure: false,
+        changeOrigin: true
+      },
+      '/config': {
+        target: 'http://localhost:4593/',
+        secure: false,
+        changeOrigin: true
+      }
+    }
 	},
 
 	module: {
